local toc, data = ...
local AddonId = toc.identifier
-- Local config options ---------------------------------------------------------
local rgMainWidth = 360
local rgMainHeight = 128

---------------------------------------------------------------------------------
local rgGadgets = {}

-- Frame Configuration Options --------------------------------------------------
local rgPlayerFrame = WT.UnitFrame:Template("rgPlayerFrame")
rgPlayerFrame.Configuration.Name = "Realm Guardians Player Frame"
rgPlayerFrame.Configuration.RaidSuitable = False
rgPlayerFrame.Configuration.FrameType = "Frame"
rgPlayerFrame.Configuration.Width = rgMainWidth
rgPlayerFrame.Configuration.Height = rgMainHeight

-- Only enable resizing if the installed version of Gadgets can handle it:
if WT.UnitFrame.EnableResizableTemplate then
      rgPlayerFrame.Configuration.Resizable = { rgMainWidth * 0.75, rgMainHeight * 0.75, rgMainWidth * 2.0, rgMainHeight * 2.0 }
end
---------------------------------------------------------------------------------

function rgPlayerFrame:Construct(options)

	table.insert(rgGadgets, self)
	
	local template =
	{
		elements = 
		{
			{
				-- Generic Element Configuration
				id="frameBackdrop", type="Frame", parent="frame", layer=0, alpha=1,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" },
				},
				visibilityBinding="id",
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=1,
				color={r=0,g=0,b=0,a=0},
				--color={r=1, g=1, b=1, a=.5},
			},
			{
				-- Generic Element Configuration
				id="frameOverlay", type="Image", parent="frame", layer=40, alpha=1,
				attach = { 
					{ point="TOPLEFT", element="frameBackdrop", targetPoint="TOPLEFT", offsetX=0, offsetY=0 },
					{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT", offsetX=0, offsetY=0 },
				},
				visibilityBinding="id", 
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/overlays/frameOverlay2.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
				
			},
			{
				-- Generic Element Configuration
				id="gcdBackdrop", type="Frame", parent="frame", layer=100, alpha=1,
				attach = { 
					{ point="TOPLEFT", element="frameBackdrop", targetPoint="CENTER", offsetX=-104, offsetY=10.5 },
					{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="CENTER", offsetX=100, offsetY=13.5},
				},
				visibilityBinding="id",
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=1,
				color={r=0,g=0,b=0,a=0}, 
			},
			{
				-- Generic Element Configuration
				id="castBackdrop", type="Frame", parent="frame", layer=0, alpha=1,
				attach = { 
					{ point="TOPLEFT", element="frameBackdrop", targetPoint="CENTER", offsetX=-104, offsetY=12 },
					{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="CENTER", offsetX=100.5, offsetY=30},
				},
				visibilityBinding="id",
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=1,
				color={r=0,g=0,b=0,a=0}, 
			},
			{
				-- Generic Element Configuration
				id="castBarBorder", type="Image", parent="frameOverlay", layer=45, alpha=1,
				attach = {
					{ point="TOPLEFT", element="frameBackdrop", targetPoint="CENTER", offsetX=-108, offsetY=4 },
					{ point="BOTTOMCENTER", element="frameBackdrop", targetPoint="BOTTOMCENTER", offsetX=-2, offsetY=0 },
				},
				visibilityBinding="castName",
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/borders/castBarBorder.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="castBarRight", type="Bar", parent="frame", layer=50, alpha=.8,
				attach = {
					{ point="TOPLEFT", element="castBackdrop", targetPoint="TOPCENTER", offsetX=-.45 },
					{ point="BOTTOMRIGHT", element="castBackdrop", targetPoint="BOTTOMRIGHT" },
				},
				growthDirection="right",
				visibilityBinding="castName",
				binding="castPercent",
				texAddon = AddonId, texFile="img/textures/Gloss.png", alpha=.5,
				colorBinding="castColor",
				backgroundColor={r=0, g=0, b=0, a=0.6},
			},
			{
				-- Generic Element Configuration
				id="castBarLeft", type="Bar", parent="frame", layer=50, alpha=.8,
				attach = {
					{ point="TOPRIGHT", element="castBarRight", targetPoint="TOPLEFT", offsetX=.45 },
					{ point="BOTTOMLEFT", element="castBackdrop", targetPoint="BOTTOMLEFT" },
				},
				growthDirection="left",
				visibilityBinding="castName",
				binding="castPercent",
				texAddon = AddonId, texFile="img/textures/Gloss.png", alpha=.5,
				colorBinding="castColor",
				backgroundColor={r=0, g=0, b=0, a=0.6}
			},
			{
				-- Generic Element Configuration
				id="statusBackdrop", type="Frame", parent="frame", layer=0, alpha=1,
				attach = { 
					{ point="CENTER", element="frameBackdrop", targetPoint="CENTER", offsetX=-1, offsetY=-16 },
				},
				visibilityBinding="id",
				width=60, height=60,
				color={r=0,g=0,b=0,a=0},
			},
			{
				-- Generic Element Configuration
				id="imgStatusOrb", type="ImageSet", parent="frame", layer=50, alpha=1, 
				attach = 
				{ 
					{ point="CENTER", element="statusBackdrop", targetPoint="CENTER", offsetX=0, offsetY=0 },
				},
				visibilityBinding="statusOrbIndex",
				texAddon=AddonId, texFile="img/statusOrbIndex.png", rows=10,cols=1,
				indexBinding="statusOrbIndex", width=60,height=60,
			},
			{
				-- Generic Element Configuration
				id="resourceBackdrop", type="Frame", parent="frame", layer=5, alpha=1,
				attach = { 
					{ point="BOTTOMLEFT", element="frame", targetPoint="CENTERLEFT", offsetX=13.5, offsetY=10},
					{ point="TOPRIGHT", element="frame", targetPoint="CENTERLEFT", offsetX=121, offsetY=-20},
				},
				visibilityBinding="id",
				colorBinding="lowManaColor",
			},
			--[[
			{
				-- Generic Element Configuration
				id="resourceAlertFrame", type="Frame", parent="frame", layer=6, alpha=1,
				attach = { 
					{ point="BOTTOMLEFT", element="frame", targetPoint="CENTERLEFT", offsetX=11, offsetY=12},
					{ point="TOPRIGHT", element="frame", targetPoint="CENTERLEFT", offsetX=123, offsetY=-24},
				},
				visibilityBinding="id",	
				colorBinding="lowManaColor",
			},
			--]]
			{
				-- Generic Element Configuration
				id="resourceBar", type="Bar", parent="frame", layer=10, alpha=.8, 
				attach = 
				{ 
					{ point="TOPLEFT", element="resourceBackdrop", targetPoint="TOPLEFT", offsetX=5.5, offsetY=5.5 },
					{ point="BOTTOMRIGHT", element="resourceBackdrop", targetPoint="BOTTOMRIGHT", offsetX=-3, offsetY=-4.25 },
				},
				--height=rgBarHeight, width=rgBarWidth,
				growthDirection="left",
				-- Type Specific Element Configuration
				binding="resourcePercent", 
				colorBinding="resourceColor",
				texAddon=AddonId, texFile="img/textures/rgBlackBar.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0.5}
			},
			{
				-- Generic Element Configuration
				id="resourceBorder", type="Image", parent="frame", layer=15, alpha=1,
				attach = 
				{ 
					{ point="BOTTOMLEFT", element="resourceBackdrop", targetPoint="BOTTOMLEFT", offsetX=-1.5, offsetY=0},
					{ point="TOPRIGHT", element="resourceBackdrop", targetPoint="TOPRIGHT", offsetX=0, offsetY=0},
				},
				--height=rgBarHeight, width=rgBarWidth,
				visibilityBinding="id",
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/borders/utilityBorder2.png", alpha=1,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="healthBackdrop", type="Frame", parent="frame", layer=5, alpha=1,
				attach = { 
					{ point="BOTTOMRIGHT", element="frame", targetPoint="CENTERRIGHT", offsetX=-13.5, offsetY=10},
					{ point="TOPLEFT", element="frame", targetPoint="CENTERRIGHT", offsetX=-121, offsetY=-20},
				},
				visibilityBinding="id",
				colorBinding="lowHealthColor",
			},
			--[[
			{
				-- Generic Element Configuration
				id="healthAlertFrame", type="Frame", parent="frame", layer=6, alpha=1,
				attach = { 
					{ point="BOTTOMRIGHT", element="frame", targetPoint="CENTERRIGHT", offsetX=-13.5, offsetY=12},
					{ point="TOPLEFT", element="frame", targetPoint="CENTERRIGHT", offsetX=-123, offsetY=-24},
				},
				visibilityBinding="id",
				colorBinding="lowHealthColor",
			},
			--]]
			{
				-- Generic Element Configuration
				id="healthBar", type="Bar", parent="frame", layer=10, alpha=.8,
				attach = 
				{ 
					{ point="TOPLEFT", element="healthBackdrop", targetPoint="TOPLEFT", offsetX=3, offsetY=5.5 },
					{ point="BOTTOMRIGHT", element="healthBackdrop", targetPoint="BOTTOMRIGHT", offsetX=-4.25, offsetY=-4.25 },
				},
				--height=rgBarHeight, width=rgBarWidth,
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="healthPercent", 
				colorBinding="healthColor",
				texAddon=AddonId, texFile="img/textures/rgBlackBar.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=.5}
			},
			{
				-- Generic Element Configuration
				id="healthBorder", type="Image", parent="frame", layer=15, alpha=1,
				attach = 
				{ 
					{ point="BOTTOMRIGHT", element="healthBackdrop", targetPoint="BOTTOMRIGHT", offsetX=1.5, offsetY=0},
					{ point="TOPLEFT", element="healthBackdrop", targetPoint="TOPLEFT", offsetX=0, offsetY=0},
				},
				--height=rgBarHeight, width=rgBarWidth,
				visibilityBinding="id",
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/borders/utilityBorder2.png", alpha=1,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="absorbBar", type="Bar", parent="frame", layer=30,
				attach = {
					{ point="TOPLEFT", element="healthBar", targetPoint={.005, 1/6} },
					{ point="BOTTOMRIGHT", element="healthBar", targetPoint={.995, 2/3} },
				},
				growthDirection="right",
				binding="absorbPercent",
				color={r=0.08, g=0.91, b=1, a=1},
				backgroundColor={r=0.08, g=0.91, b=1, a=0}
			},
			--[[
			{
				-- Generic Element Configuration
				id="labelName", type="Label", parent="frame", layer=100, alpha=1,
				attach = {
					{ point="BOTTOMLEFT", element="frameBackdrop", targetPoint="BOTTOMLEFT", offsetX=12, offsetY=8 },
				},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{name}", default="", font="Bazooka", fontSize=14, outline=true
			},
			--]]
			{
				-- Generic Element Configuration
				id="healthLabel", type="Label", parent="frame", layer=100, alpha=1,
				attach = {
					{ point="CENTER", element="healthBorder", targetPoint="CENTER", offsetX=0, offsetY=0 },
				},
				visibilityBinding="health",
				-- Type Specific Element Configuration
				text="{health}  ({healthPercent}%)", default="", font="ArmWrestler", fontSize=11, outline=true, 
			},
			{
				-- Generic Element Configuration 
				id="resourceLabel", type="Label", parent="frame", layer=100, alpha=1,
				attach = {
					{ point="CENTER", element="resourceBorder", targetPoint="CENTER", offsetX=0, offsetY=0 },
				},
				visibilityBinding="resource",
				-- Type Specific Element Configuration
				text="({resourcePercent}%)  {resource}", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler", fontSize=11, outline=true, 
			},
			{
				-- Generic Element Configuration
				id="frameRole", type="Frame", parent="frame", layer=20, alpha=1,
				attach = {{ point="BOTTOMLEFT", element="resourceBorder", targetPoint="TOPRIGHT", offsetX=-13.5, offsetY=20 }},
				height=28, width = 28,
				color={r=0,g=0,b=0,a=0}, 
			},
			{
				id="imgRole", type="MediaSet", parent="frame", layer=60, alpha=1,
				attach = {{ point="CENTER", element="frameRole", targetPoint="CENTER" }},
				width = 24, height = 24,
				visibilityBinding="role",
				nameBinding="role", 
				names = { ["tank"] = "Tank", ["heal"] = "Heal", ["dps"] = "DPS", ["support"] = "Support" },
			},		
			{
				id="imgPVP", type="MediaSet", parent="frame", layer=100, alpha=1,
				attach = {{ point="CENTERRIGHT", element="statusBackdrop", targetPoint="CENTERLEFT", offsetX=1, offsetY=-6 }}, 
				width=24, height=24,
				nameBinding="pvpFaction",
				names = 
				{
					["defiant"] = "FactionDefiant",
					["guardian"] = "FactionGuardian",
				},
			},
			{
				id="imgConquest", type="MediaSet", parent="frame", layer=100, 
				attach = {{ point="CENTERLEFT", element="statusBackdrop", targetPoint="CENTERRIGHT", offsetX=-4, offsetY=-7 }}, 
				width=28, height=28,
				nameBinding="cqFaction",
				names = 
				{
					["nightfall"] = "factionNightfall",
					["oathsworn"] = "factionOathsworn",
					["dominion"] = "factionDominion",
				},
			},
			{
				-- Generic Element Configuration
				id="frameCharges", type="Frame", parent="frame", layer=20, alpha=1,
				attach = {{ point="BOTTOMRIGHT", element="healthBorder", targetPoint="TOPLEFT", offsetX=13.5, offsetY=20 }},
				visibilityBinding="planarMax",
				backgroundColor={r=0, g=0, b=0, a=0.8}, height=28, width=28,
			},
			{
				-- Generic Element Configuration
				id="imgCharge", type="Image", parent="frame", layer=60, alpha=1,
				attach = {{ point="CENTER", element="frameCharges", targetPoint="CENTER" }},
				texAddon=AddonId, texFile="img/icons/planarIcon.png", 
				backgroundColor={r=0, g=0, b=0, a=0}, height=22, width=22,
			},
			{
				-- Generic Element Configuration
				id="chargeLabel", type="Label", parent="frame", layer=100,
				attach = {{ point="CENTER", element="imgCharge", targetPoint="CENTER", offsetX=0, offsetY=0 }},
				text="{planar}", font="Bazooka", fontSize=14, outline=true,
			},
			{
				-- Generic Element Configuration
				id="frameVitality", type="Frame", parent="frame", layer=50, alpha=1,
				attach = {{ point="CENTERRIGHT", element="frameBackdrop", targetPoint="CENTERRIGHT", offsetX=-34, offsetY=22 }},
				color={r=0,g=0,b=0,a=0}, height=20, width=20
			},
			{
				-- Generic Element Configuration
				id="txtVitality", type="Label", parent="frame", layer=100,
				attach = {{ point="CENTER", element="frameVitality", targetPoint="CENTER" }},
				text="{vitality}%", visibilityBinding="vitalityDisplay", font="Bazooka", fontSize=12, 
			},
			{
				-- Generic Element Configuration
				id="imgWingVitality", type="ImageSet", parent="frame", layer=0, alpha=.8,
				attach = {
					{ point="CENTER", element="frameOverlay", targetPoint="CENTER", offsetX=0, offsetY=8 },
				},
				texAddon=AddonId, texFile="img/wingVitalityIndex.png", rows=10,cols=1,
				indexBinding="wingVitalityIndex", width=370,height=90,
			},
			--[[
			{
				-- Generic Element Configuration
				id="imgMark", type="MediaSet", parent="frame", layer=40,
				attach = {{ point="BOTTOMCENTER", element="frameBackdrop", targetPoint="TOPCENTER", offsetX=-1, offsetY=-12 }},
				width = 24, height = 24,
				nameBinding="mark",
				names = 
				{
						["1"] = "riftMark01",
						["2"] = "riftMark02",
						["3"] = "riftMark03",
						["4"] = "riftMark04",
						["5"] = "riftMark05",
						["6"] = "riftMark06",
						["7"] = "riftMark07",
						["8"] = "riftMark08",
						["9"] = "riftMark09",
						["10"] = "riftMark10",
						["11"] = "riftMark11",
						["12"] = "riftMark12",
						["13"] = "riftMark13",
						["14"] = "riftMark14",
						["15"] = "riftMark15",
						["16"] = "riftMark16",
						["17"] = "riftMark17",
				},
				visibilityBinding="mark",alpha=1.0,
			},
			--]]
			{
			    id="statusText", type="Label", parent="frame", layer=60,
				attach = {{ point="CENTER", element="statusBackdrop", targetPoint="CENTER", offsetX=-1, offsetY=0 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{selfStatus}", default="", font="Bazooka", fontSize=16,
			},
			{
				id="buffPanelDebuffs", type="BuffPanel", parent="frame", layer=90,
				attach = {{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT", offsetX=-10, offsetY=-2 }},
				rows=1, cols=6, iconSize=20, iconSpacingHorizontal=1, iconSpacingVertical=1, borderThickness=0.5, 
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=false, acceptHighPriorityBuffs=false, acceptCriticalPriorityBuffs=false,
				acceptLowPriorityDebuffs=true, acceptMediumPriorityDebuffs=true, acceptHighPriorityDebuffs=true, acceptCriticalPriorityDebuffs=true,
				growthDirection = "left_up", borderColor={r=0,g=0,b=0,a=1},
				timerSize=10, timerOffsetX=0, timerOffsetY=16,
				stackSize=10, stackOffsetX=0, stackOffsetY=0, stackBackgroundColor={r=0,g=0,b=0,a=0.3},
				sweepOverlay=false,
			},
			{
				-- Generic Element Configuration
				id="labelCast", type="Label", parent="frame", layer=100, alpha=1,
				attach = {{ point="CENTER", element="castBackdrop", targetPoint="CENTER", offsetX=-20, offsetY=0 }},
				visibilityBinding="castName",
				text="{castName}", default="", color={r=1,g=1,b=1,a=1}, font="Bazooka", fontSize=12, outline=true,
			},
			{
				id="labelCastTime", type="Label", parent="frame", layer=100, alpha=1,
				attach = 
				{
					{ point="CENTERRIGHT", element="castBackdrop", targetPoint="CENTERRIGHT", offsetX=-2, offsetY=0 },
				},
				visibilityBinding="castName",
				text="{castTime}", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler",  fontSize=11, 
			},
			{
				id="comboBackdrop", type="Frame", parent="frame", layer=0, alpha=1,
				attach = 
				{ 
					{ point="CENTER", element="frameBackdrop", targetPoint="CENTER", offsetX=0, offsetY=-60 },
				},
				visibilityBinding="id",
				width=200, height=60,
				color={r=0,g=0,b=0,a=0}, 
			},
			{
				id="imgCombo", type="ImageSet", parent="frame", layer=10, alpha=1,
				attach = 
				{
					{ point="CENTER", element="comboBackdrop", targetPoint="CENTER", offsetX=-0, offsetY=-0 },
				},
				indexBinding="combo", rows=6, cols=1,
				visibilityBinding="combo", width=200, height=60,
				texAddon=AddonId, texFile="img/bloodComboIndex.png", alpha=1,
			},
			{
				-- Mage Charge Bar Frame
				id="chargeBackdrop", type="Frame", parent="frame", layer=0, alpha=1,
				attach = 
				{ 
					{ point="CENTER", element="frameBackdrop", targetPoint="CENTER", offsetX=-1, offsetY=-41 },
				},
				visibilityBinding="id",
				width=150, height=42,
				color={r=0,g=0,b=0,a=0}, 
			},
			{
				-- Mage Right Charge Bar
				id="chargeBarRight", type="Bar", parent="frame", layer=10, alpha=.8,
				attach = 
				{
					{ point="TOPLEFT", element="chargeBackdrop", targetPoint="TOPCENTER", offsetX=-.25, offsetY=3.5 },
					{ point="BOTTOMRIGHT", element="chargeBackdrop", targetPoint="BOTTOMRIGHT", offsetX=-2, offsetY=0 },
				},
				growthDirection="right",
				binding="chargePercent", color={r=0,g=0,b=0,a=0},
				texAddon = AddonId, texFile="img/textures/chargeArcRight.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0}
			},
			{
				-- Mage Left Charge Bar
				id="chargeBarLeft", type="Bar", parent="frame", layer=10, alpha=.8,
				attach = 
				{
					{ point="TOPRIGHT", element="chargeBarRight", targetPoint="TOPLEFT", offsetX=.25, offsetY=.25 },
					{ point="BOTTOMLEFT", element="chargeBackdrop", targetPoint="BOTTOMLEFT", offsetX=3.5, offsetY=0 },
				},
				growthDirection="left",
				binding="chargePercent", color={r=0,g=0,b=0,a=0},
				texAddon = AddonId, texFile="img/textures/chargeArcLeft.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0}
			},
			{
				-- Mage Charge Border
				id="chargeBarBorder", type="Image", parent="frame", layer=5, alpha=1,
				attach = 
				{
					{ point="TOPLEFT", element="chargeBackdrop", targetPoint="TOPLEFT", offsetX=-3.5, offsetY=-1.5 },
					{ point="BOTTOMRIGHT", element="chargeBackdrop", targetPoint="BOTTOMRIGHT", offsetX=1.5, offsetY=0 },
				},
				visibilityBinding="chargeText", 
				texAddon=AddonId, texFile="img/borders/chargeArcBorder.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Mage Charge Label
				id="labelCharge", type="Label", parent="frame", layer=80, 
				attach = 
				{
					{ point="CENTER", element="chargeBarBorder", targetPoint="TOPCENTER", offsetX=0, offsetY=14 },
				},
				visibilityBinding="chargeText",
				text="{charge}", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler",  fontSize=12, 
			},
		}
	}
	
	if WT.UnitFrame.EnableResizableTemplate then
            WT.UnitFrame.EnableResizableTemplate(self, rgMainWidth, rgMainHeight, template.elements)
    end
	
	local animations =
		{
            {
				-- Low Health Animation
                trigger = "healthWarning",
                duration = .80,
                loopCount = 0,
                onStart =
					function(elements)
						elements.healthBackdrop:SetAlpha(1.0)
                        end,
					onTick =
                        function(elements, elapsed, progress)
                            elements.healthBackdrop:SetAlpha((math.sin(progress * (2.0 * math.pi)) + 1.0) / 2.0)
                        end,             
					onFinish =
                        function(elements)
                            elements.healthBackdrop:SetAlpha(1.0)
                        end,             
            },
			{
				-- Low Mana Animation
                trigger = "manaWarning",
                duration = .80,
                loopCount = 0,
                onStart =
					function(elements)
						elements.resourceBackdrop:SetAlpha(1.0)
                        end,
					onTick =
                        function(elements, elapsed, progress)
                            elements.resourceBackdrop:SetAlpha((math.sin(progress * (2.0 * math.pi)) + 1.0) / 2.0)
                        end,             
					onFinish =
                        function(elements)
                            elements.resourceBackdrop:SetAlpha(1.0)
                        end,             
            },
			{
				-- Low Vitality Animation
                trigger = "vitalityWarning",
                duration = 3.0,
                loopCount = 0,
                onStart =
					function(elements)
						elements.imgWingVitality:SetAlpha(1.0)
                        end,
					onTick =
                        function(elements, elapsed, progress)
                            elements.imgWingVitality:SetAlpha((math.sin(progress * (2.0 * math.pi)) + 1.5) / 3.0)
                        end,             
					onFinish =
                        function(elements)
                            elements.imgWingVitality:SetAlpha(1.0)
                        end,             
            },
			{
				-- Full Combo Points Animation
                trigger = "comboWarning",
                duration = 1.50,
                loopCount = 0,
                onStart =
					function(elements)
						elements.imgCombo:SetAlpha(0.5)
                        end,
					onTick =
                        function(elements, elapsed, progress)
                            elements.imgCombo:SetAlpha((math.sin(progress * (2.0 * math.pi)) + 1.5) / 3.0)
                        end,             
					onFinish =
                        function(elements)
                            elements.imgCombo:SetAlpha(1)
                        end,             
            },
      }
     
      if WT.UnitFrame.EnableAnimation then
            WT.UnitFrame.EnableAnimation(self, animations)
      end
     
	
	for idx,element in ipairs(template.elements) do
        local showElement = true
        if not options.showAbsorb and (element.id=="barAbsorbTop" or element.id=="barAbsorbRight" or 
        element.id=="barAbsorbLeft" or element.id=="barAbsorbBottom" or element.id=="labelHealthAndAbsorbPercent") then showElement = false end
        if showElement then
            self:CreateElement(element)
        end 
    end

    self:AddGCDTrack("gcdBackdrop")
    
    self:SetSecureMode("restricted")
    self:SetMouseoverUnit(self.UnitSpec)
    self.Event.LeftClick = "target @" .. self.UnitSpec
    self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
end

function rgPlayerFrame:AddGCDTrack(elementId)

    local attachToElement = self.Elements[elementId]
    
    self.gcdTrack = UI.CreateFrame("Frame", "gcdTrack", attachToElement)
    self.gcdTrack:SetLayer(100)
    self.gcdTrack:SetPoint("BOTTOMRIGHT", attachToElement, "TOPCENTER")
	self.gcdTrack:SetPoint("TOPLEFT", attachToElement, "TOPLEFT", 0, -3)
    
    self.gcdSpark = UI.CreateFrame("Texture", "gcdSpark", self.gcdTrack)
    self.gcdSpark:SetTexture(AddonId, "img/spark/sparkYellow.png")
    self.gcdSpark:SetPoint("CENTER", self.gcdTrack, 1, 0.5, 0.0, 1)
    self.gcdSpark:SetHeight(20)
    self.gcdSpark:SetWidth(20)
    self.gcdSpark:SetAlpha(0.8)
    self.gcdSpark:SetVisible(false)

    self.gcdTrack2 = UI.CreateFrame("Frame", "gcdTrack2", attachToElement)
    self.gcdTrack2:SetLayer(100)
    self.gcdTrack2:SetPoint("BOTTOMLEFT", attachToElement, "TOPCENTER")
	self.gcdTrack2:SetPoint("TOPRIGHT", attachToElement, "TOPRIGHT", 0, -3)
    
    self.gcdSpark2 = UI.CreateFrame("Texture", "gcdSpark2", self.gcdTrack)
    self.gcdSpark2:SetTexture(AddonId, "img/spark/sparkYellow.png")
    self.gcdSpark2:SetPoint("CENTER", self.gcdTrack2, 0.0, 0.5, 1, 1)
    self.gcdSpark2:SetHeight(20)
    self.gcdSpark2:SetWidth(20)
    self.gcdSpark2:SetAlpha(0.8)
    self.gcdSpark2:SetVisible(false)
    
    -- Hide the tracks (need to use alpha so that the spark doesn't also get hidden)
    self.gcdTrack:SetBackgroundColor(0,0,0, 0.0)
    self.gcdTrack2:SetBackgroundColor(0,0,0, 0.0)
    
end

-- Add handling for a GCD spark, as used in the castbar gadget.

local trackingCD = nil
local trackingStart = nil
local trackingDuration = nil

local function cdBegin(list)
    for abilityId, cooldown in pairs(list) do
        if (cooldown >= 0.9) and (cooldown <= 1.51) then
            trackingCD = abilityId
            trackingStart = Inspect.Time.Frame()
            trackingDuration = cooldown
            for idx, castbar in ipairs(rgGadgets) do
                castbar.gcdSpark:SetVisible(true)
                castbar.gcdSpark2:SetVisible(true)
            end
            break
        end
    end
end

local function cdEnd(list)
    if trackingCD and list[trackingCD] then
        -- GCD has ended
        trackingCD = nil
        for idx, castbar in ipairs(rgGadgets) do
            castbar.gcdSpark:SetVisible(false)
            castbar.gcdSpark2:SetVisible(false)
        end
    end
end

local function cdUpdate()
    if not trackingCD then return end
    local elapsed = Inspect.Time.Frame() - trackingStart
    local percent = elapsed / trackingDuration
    if percent > 1.0 then percent = 1.0 end
    for idx, castBar in ipairs(rgGadgets) do
        castBar.gcdSpark:SetPoint("CENTER", castBar.gcdTrack, 1.0 - percent, 0.5, 0.0, 1)   
        castBar.gcdSpark2:SetPoint("CENTER", castBar.gcdTrack2, percent, 0.5, 1, 1) 
    end 
end

table.insert(Event.Ability.New.Cooldown.Begin, { cdBegin, AddonId, AddonId .. "_CooldownBegin" })
table.insert(Event.Ability.New.Cooldown.End, { cdEnd, AddonId, AddonId .. "_CooldownEnd"  })
table.insert(Event.System.Update.Begin, { cdUpdate, AddonId, AddonId .. "_CooldownUpdate" })