local toc, data = ...
local AddonId = toc.identifier
-- Local config options ---------------------------------------------------------
local rgRaidFrameWidth = 110
local rgRaidFrameHeight = 50

---------------------------------------------------------------------------------

-- Frame Configuration Options --------------------------------------------------
local rgRaidFrame = WT.UnitFrame:Template("rgRaidFrame")
rgRaidFrame.Configuration.Name = "Realm Guardians Raid Frames"
rgRaidFrame.Configuration.RaidSuitable = true
rgRaidFrame.Configuration.FrameType = "Frame"
rgRaidFrame.Configuration.Width = rgRaidFrameWidth
rgRaidFrame.Configuration.Height = rgRaidFrameHeight

-- Only enable resizing if the installed version of Gadgets can handle it:
if WT.UnitFrame.EnableResizableTemplate then
      rgRaidFrame.Configuration.Resizable = { rgRaidFrameWidth * 0.75, rgRaidFrameHeight * 0.75, rgRaidFrameWidth * 2.0, rgRaidFrameHeight * 2.0 }
end
---------------------------------------------------------------------------------

function rgRaidFrame:Construct(options)

	local template =
	{
		elements = 
		{
			{
				id="frameBackdrop", type="Image", parent="frame", layer=0, alpha=1,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=.5, offsetY=.5 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-.5, offsetY=-.5 },
				},
				visibilityBinding="id", 
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/borders/raidBackdrop.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				id="aggroBorder", type="Image", parent="frame", layer=2, alpha=1,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=-6, offsetY=-6 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=6, offsetY=6 },
				}, 				
				visibilityBinding="unitAggro",
				texAddon = AddonId, texFile = "img/borders/aggroBorder.png", alpha=1,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				id="innerFrame", type="Image", parent="frame", layer=8, alpha=1,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=.5, offsetY=.5 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-.5, offsetY=-.5 } 
				}, 	
				visibilityBinding="id",
				texAddon = AddonId, texFile = "img/borders/rgInnerFrame.png", alpha=1,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				id="barHealth", type="Bar", parent="frame", layer=30, alpha=.8,
				attach = 
				{
					{ point="TOPLEFT", element="frameBackdrop", targetPoint="TOPLEFT", offsetX=22.25, offsetY=16.75 },
					{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT", offsetX=-10, offsetY=-8.75 },
				},
				visibilityBinding="rgRaidCleanse",
				growthDirection="right",
				binding="healthPercent", colorBinding="callingColor", color={r=0, g=0, b=0, a=0},
				texAddon=AddonId, texFile="img/textures/rgBlackBar.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="barCleanse", type="Bar", parent="frame", layer=40, alpha=.6,
				attach = {
					{ point="TOPLEFT", element="barHealth", targetPoint="TOPLEFT", offsetY=7 },
					{ point="BOTTOMRIGHT", element="barHealth", targetPoint="BOTTOMRIGHT" },
				},
				visibilityBinding="cleanseBar",
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="healthPercent", color={r=1, g=.41, b=.71, a=.6},
				--texAddon=AddonId, texFile="img/textures/rgBarCleanse.png", alpha=.9,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				id="barResourceRight", type="Bar", parent="frame", layer=50, alpha=.8,
				attach = 
				{
					{ point="TOPLEFT", element="barHealth", targetPoint="TOPCENTER", offsetX=-.0005, offsetY=.5 },
					{ point="BOTTOMRIGHT", element="barHealth", targetPoint="TOPRIGHT", offsetX=-2, offsetY=6 },
				},
				visibilityBinding="id",
				growthDirection="right",
				binding="resourcePercent", --colorBinding="resourceColor",
				--texAddon=AddonId, texFile="img/textures/rgBar.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0}
			},
			{
				id="barResourceLeft", type="Bar", parent="frame", layer=50, alpha=.8,
				attach = 
				{
					{ point="TOPRIGHT", element="barHealth", targetPoint="TOPCENTER", offsetX=.0005, offsetY=.5 },
					{ point="BOTTOMLEFT", element="barHealth", targetPoint="TOPLEFT", offsetX=2, offsetY=6 },
				},
				visibilityBinding="id",
				growthDirection="left",
				binding="resourcePercent", --colorBinding="resourceColor",
				--texAddon=AddonId, texFile="img/textures/rgBar.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0}
			},
			{
				id="barAbsorb", type="Bar", parent="frame", layer=50,
				attach = {
					{ point="BOTTOMLEFT", element="barHealth", targetPoint="BOTTOMLEFT", offsetX=3, offsetY=-1 },
					{ point="TOPRIGHT", element="barHealth", targetPoint="BOTTOMRIGHT", offsetX=-3, offsetY=-7 },
				},
				growthDirection="right",
				binding="absorbPercent", color={r=0,g=1,b=1,a=1},
				--texAddon=AddonId, texFile="img/textures/gradientLeft.png", alpha=.5,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="labelName", type="Label", parent="frame", layer=150, alpha=1,
				attach = {{ point="BOTTOMCENTER", element="barHealth", targetPoint="TOPCENTER", offsetX=-2, offsetY=4 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{nameShort}", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler", fontSize=9, 
			},
			{
				-- Generic Element Configuration
				id="labelRange", type="Label", parent="frame", layer=150, alpha=1,
				attach = {{ point="CENTER", element="barHealth", targetPoint="CENTER", offsetX=0, offsetY=1.5 }},
				visibilityBinding="rangeText",
				text="{range} m", default="", colorBinding="rangeColor", font="ArmWrestler", fontSize=10, 
			},
			{
				-- Generic Element Configuration
				id="frameRole", type="Frame", parent="frame", layer=40, alpha=1,
				attach = {{ point="TOPLEFT", element="frameBackdrop", targetPoint="TOPLEFT", offsetX=8, offsetY=6.5 }}, 
				width = 14, height = 14,
				color={r=0,g=0,b=0,a=0}, 
			},
			{
				id="imgRole", type="MediaSet", parent="frame", layer=80,
				attach = {{ point="CENTER", element="frameRole", targetPoint="CENTER" }},
				width = 13, height = 13,
				visibilityBinding="role",
				nameBinding="role", 
				names = { ["tank"] = "rgRaidTank", ["heal"] = "rgRaidHeal", ["dps"] = "rgRaidDPS", ["support"] = "rgRaidSupport" },
			},
			{
				-- Generic Element Configuration
				id="frameCharges", type="Frame", parent="frame", layer=40, alpha=1,
				attach = {{ point="TOPRIGHT", element="imgRole", targetPoint="BOTTOMRIGHT", offsetX=-2, offsetY=0 }},
				width = 10, height = 10,
				color={r=0,g=0,b=0,a=0}, 
			},
			{
				-- Generic Element Configuration
				id="chargeLabel", type="Label", parent="frame", layer=80,
				attach = {{ point="CENTER", element="frameCharges", targetPoint="CENTER", offsetX=0, offsetY=0 }},
				text="{planar}", color={r=0,g=1,b=1,a=1}, font="ArmWrestler", fontSize=9,
			},
			{
				-- Generic Element Configuration
				id="frameVitality", type="Frame", parent="frame", layer=40, alpha=1,
				attach = {{ point="TOPRIGHT", element="frameCharges", targetPoint="BOTTOMRIGHT", offsetX=2.5, offsetY=-1.50 }},
				width = 14, height = 14,
				color={r=0,g=0,b=0,a=0}, 
			},
			{
				-- Generic Element Configuration
				id="imgVitality", type="ImageSet", parent="frame", layer=60, alpha=1,
				attach = {{ point="CENTER", element="frameVitality", targetPoint="CENTER" }},
				texAddon=AddonId, texFile="img/wtVitality.png", rows=10,cols=1,
				indexBinding="vitalityIndex", visibilityBinding="vitalityIndex", width=13, height=13,
			},
			{
				-- Generic Element Configuration
				id="txtVitality", type="Label", parent="frame", layer=80,
				attach = {{ point="CENTER", element="imgVitality", targetPoint="CENTER" }},
				text="{vitality}", visibilityBinding="vitalityIndex", font="ArmWrestler", fontSize=9,
			},
			{
				-- Generic Element Configuration
				id="labelStatus", type="Label", parent="frame", layer=150,
				attach = {{ point="CENTER", element="barHealth", targetPoint="CENTER", offsetX=-2, offsetY=3 }},
				visibilityBinding="rgStatus",
				text="{rgStatus}", default="", font="Bazooka", fontSize=10
			},
			{
			    id="imgMark", type="MediaSet", parent="frame", layer=60,
			    attach = {{ point="CENTERLEFT", element="barHealth", targetPoint="CENTERLEFT", offsetX=2.5, offsetY=2 }},
			    width = 16, height = 16,
			    nameBinding="mark",
			    names = 
			    {
			        ["1"] = "riftMark01_mini",
			        ["2"] = "riftMark02_mini",
			        ["3"] = "riftMark03_mini",
			        ["4"] = "riftMark04_mini",
			        ["5"] = "riftMark05_mini",
			        ["6"] = "riftMark06_mini",
			        ["7"] = "riftMark07_mini",
			        ["8"] = "riftMark08_mini",
			        ["9"] = "riftMark09_mini",
			        ["10"] = "riftMark10_mini",
			        ["11"] = "riftMark11_mini",
			        ["12"] = "riftMark12_mini",
			        ["13"] = "riftMark13_mini",
			        ["14"] = "riftMark14_mini",
			        ["15"] = "riftMark15_mini",
			        ["16"] = "riftMark16_mini",
			        ["17"] = "riftMark17_mini",
			    },
			    visibilityBinding="mark", alpha=1.0,
			},
			{
				-- Generic Element Configuration
				id="imgReady", type="ImageSet", parent="frame", layer=50,
				attach = {{ point="CENTERRIGHT", element="barHealth", targetPoint="CENTERRIGHT", offsetX=-2, offsetY=1.5 }}, 
				 width = 20, height = 20,
				 visibilityBinding="readyStatus",
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/icons/ncReady.png", nameBinding="readyStatus", cols=1, rows=2, 
				names = { ["ready"] = 0, ["notready"] = 1 }, defaultIndex = "hide"
			},
			{
				-- Generic Element Configuration
				id="buffPanelDebuffs", type="BuffPanel", parent="frame", layer=60,
				attach = {{ point="TOPRIGHT", element="barHealth", targetPoint="BOTTOMRIGHT", offsetX=6, offsetY=-6 }},
				-- Type Specific Element Configuration
				rows=1, cols=3, iconSize=14, iconSpacing=.75, borderThickness=.5, 
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=false, acceptHighPriorityBuffs=false, acceptCriticalPriorityBuffs=false,
				acceptLowPriorityDebuffs=false, acceptMediumPriorityDebuffs=true, acceptHighPriorityDebuffs=true, acceptCriticalPriorityDebuffs=true,
				growthDirection = "left_up", sweepOverlay=false,
			},
			{
				-- Generic Element Configuration
				id="frameBlocked", type="Frame", parent="frame", layer=110, 
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=1, offsetY=1, },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-1, offsetY=-1, }
				},
				visibilityBinding="blockedOrOutOfRange",
				color={r=0,g=0,b=0,a=0.6},
			}, 
			{
				-- Generic Element Configuration
				id="targetedBorder", type="Image", parent="frame", layer=120, alpha=1,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=.5, offsetY=.5, },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-.5, offsetY=-.5, } 
				}, 				
				visibilityBinding="unitTargeted",
				texAddon = AddonId, texFile = "img/borders/targetedBorder.png", alpha=1,
				backgroundColor={r=0, g=0, b=0, a=0},
			}, 
		}
	}
	
	if WT.UnitFrame.EnableResizableTemplate then
            WT.UnitFrame.EnableResizableTemplate(self, rgRaidFrameWidth, rgRaidFrameHeight, template.elements)
    end
	
	local animations =
		{
            {
				-- Aggro Warning Animation
                trigger = "aggroWarning",
                duration = .80,
                loopCount = 0,
                onStart =
					function(elements)
						elements.aggroBorder:SetAlpha(.5)
                        end,
					onTick =
                        function(elements, elapsed, progress)
                            elements.aggroBorder:SetAlpha((math.sin(progress * (2.0 * math.pi)) + 1.5) / 3.0)
                        end,             
					onFinish =
                        function(elements)
                            elements.aggroBorder:SetAlpha(1.0)
                        end,             
            },
      }
     
      if WT.UnitFrame.EnableAnimation then
            WT.UnitFrame.EnableAnimation(self, animations)
      end
	
	for idx,element in ipairs(template.elements) do
		if not options.showAbsorb and element.id == "barAbsorb" then 
			-- showElement = false
		else 
			self:CreateElement(element)
		end
	end
	
	self:CreateBinding("calling", self, 		
		function(frame, newCalling)	
			local img = self.Elements.barResourceRight.Image
			if not newCalling then return end
			if newCalling == "warrior" then img:SetTexture(AddonId, "img/textures/neonPowerRight.png") end
			if newCalling == "rogue" then img:SetTexture(AddonId, "img/textures/neonEnergyRight.png") end
			if newCalling == "mage" then img:SetTexture(AddonId, "img/textures/neonManaRight.png") end		
			if newCalling == "cleric" then img:SetTexture(AddonId, "img/textures/neonManaRight.png") end	
	end	)
	
	self:CreateBinding("calling", self, 		
		function(frame, newCalling)	
			local img = self.Elements.barResourceLeft.Image
			if not newCalling then return end
			if newCalling == "warrior" then img:SetTexture(AddonId, "img/textures/neonPowerLeft.png") end
			if newCalling == "rogue" then img:SetTexture(AddonId, "img/textures/neonEnergyLeft.png") end
			if newCalling == "mage" then img:SetTexture(AddonId, "img/textures/neonManaLeft.png") end		
			if newCalling == "cleric" then img:SetTexture(AddonId, "img/textures/neonManaLeft.png") end	
	end	)
	--[[
	self:CreateBinding("calling", self, 		
		function(frame, newCalling)	
			local img = self.Elements.barHealth.Image
			if not newCalling then return end
			if newCalling == "warrior" then img:SetTexture(AddonId, "img/textures/darkWarrior.png") end
			if newCalling == "rogue" then img:SetTexture(AddonId, "img/textures/darkRogue.png") end
			if newCalling == "mage" then img:SetTexture(AddonId, "img/textures/darkMage.png") end		
			if newCalling == "cleric" then img:SetTexture(AddonId, "img/textures/darkCleric.png") end	
	end	)
	--]]
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self:SetMouseMasking("limited")
	if options.clickToTarget then
		self.Event.LeftClick = "target @" .. self.UnitSpec
	end
	if options.contextMenu then
		self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
	end

end

