local toc, data = ...
local AddonId = toc.identifier

------------------------------------------------------------------------------------------------------------
-- Local config options 
------------------------------------------------------------------------------------------------------------
local rgTargetFrameWidth = 190
local rgTargetFrameTopBarHeight = 38
local rgTargetFrameBottomBarHeight = 18
local rgTargetFrameHeight = rgTargetFrameTopBarHeight + rgTargetFrameBottomBarHeight 

------------------------------------------------------------------------------------------------------------
-- Frame Configuration Options 
------------------------------------------------------------------------------------------------------------
local rgTargetFrame = WT.UnitFrame:Template("rgTargetFrame")
rgTargetFrame.Configuration.Name = "Realm Guardians Target Frame"
rgTargetFrame.Configuration.RaidSuitable = False
rgTargetFrame.Configuration.FrameType = "Frame"
rgTargetFrame.Configuration.Width = rgTargetFrameWidth 
rgTargetFrame.Configuration.Height = rgTargetFrameHeight

-- Only enable resizing if the installed version of Gadgets can handle it:
if WT.UnitFrame.EnableResizableTemplate then
      rgTargetFrame.Configuration.Resizable = { rgTargetFrameWidth * 0.75, rgTargetFrameHeight * 0.75, rgTargetFrameWidth * 2.0, rgTargetFrameHeight * 2.0 }
end
------------------------------------------------------------------------------------------------------------
-- Override the buff filter to hide some buffs
------------------------------------------------------------------------------------------------------------
local buffPriorities = 
{
	["Touched by Greed"] = 0,
	["Track Wood"] = 0,
	["Track Ore"] = 0,
	["Track Plants"] = 0,
	["Track Fish"] = 0,
	["Rested"] = 0,
	["Prismatic Glory"] = 0,
	["Looking for Group Cooldown"] = 0,
	["Feast of Domination"] = 0,
	["Feast of Vengeance"] = 0,
	["Poisonous Coating"] = 0,
	["Remarkable Whetstone"] = 0,
	["Luminescent Powerstone"] = 0,
	["Comfortable Riding Chaps"] = 0,
	["Comfortable Insoles"] = 0,
	["Quantum Sight"] = 0,
	["Planar Protection"] = 0,
	["Stellar Brightsurge Vial"] = 0,
	["Stellar Powersurge Vial"] = 0,
	["Stellar Enduring Vial"] = 0,
	["Stellar Fortified Vial"] = 0,
	["Summon Banner of Rage"] = 0,
	["Summon Banner of Concentration"] = 0,
	["Summon Banner of Fortitude"] = 0,
	["Summon Banner of Protection"] = 0,
	["Fanfare of Vigor"] = 0,
	["Fanfare of Power"] = 0,
	["Ascended Courage"] = 0,
	["Planar Protection"] = 0,
	["Heroic Resolve"] = 0,
	["Empowerment: Mounted Swiftness"] = 0,
	["Empowerment: Offense"] = 0,
	["Empowerment: Healing"] = 0,
	["Thick Armor Plating"] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	
}
function rgTargetFrame:GetBuffPriority(buff)
	if not buff then return 2 end
	return buffPriorities[buff.name] or 2
end

------------------------------------------------------------------------------------------------------------
--
------------------------------------------------------------------------------------------------------------
function rgTargetFrame:Construct(options)

	local template =
	{
		elements = 
		{
			{
				-- Generic Element Configuration
				id="frameBackdrop", type="Frame", parent="frame", layer=0,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" }, 
				},
				visibilityBinding="id", color={r=0,g=0,b=0,a=.6},
			},
			{
				-- Generic Element Configuration
				id="frameBorder", type="MediaSet", parent="frame", layer=4, alpha=1,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=-5, offsetY=-7 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=4.5, offsetY=14 },
				},
				visibilityBinding="idElite", 
				nameBinding="idElite",
				names = 
				{
					["eliteBorder"] = "eliteBorder",
					["normalBorder"] = "normalBorder",
				},
			},
			{
				-- Generic Element Configuration
				id="bossBorder", type="Image", parent="frame", layer=2, alpha=1,
				attach = { 
					{ point="TOPLEFT", element="frameBorder", targetPoint="TOPLEFT", offsetX=-4.5, offsetY=-8 },
					{ point="BOTTOMRIGHT", element="frameBorder", targetPoint="BOTTOMRIGHT", offsetX=4, offsetY=7},
				},
				visibilityBinding="idBoss", 
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/borders/bossBorder.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="rareBorder", type="Image", parent="frame", layer=2, alpha=1,
				attach = { 
					{ point="TOPLEFT", element="frameBorder", targetPoint="TOPLEFT", offsetX=-6, offsetY=-10 },
					{ point="BOTTOMRIGHT", element="frameBorder", targetPoint="BOTTOMRIGHT", offsetX=6.5, offsetY=9 },
				},
				visibilityBinding="guaranteedLoot", 
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/borders/rareBorder.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},	
			{
				-- Generic Element Configuration
				id="frameBlocked", type="Frame", parent="frame", layer=100, visibilityBinding="blockedOrOutOfRange",
				color={r=0,g=0,b=0},alpha=0.6,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=0, offsetY=0 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=0, offsetY=0 } 
				},
			}, 
			{
				-- Generic Element Configuration
				id="barResource", type="Bar", parent="frame", layer=15, alpha=.8,
				attach = {
					{ point="BOTTOMLEFT", element="frame", targetPoint="BOTTOMLEFT", offsetX=4, offsetY=.25 },
					{ point="RIGHT", element="frame", targetPoint="RIGHT", offsetX=-4 },
				},
				-- visibilityBinding="id",
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="resourcePercent", height=rgTargetFrameBottomBarHeight, colorBinding="resourceColor",
				texAddon=AddonId, texFile="img/textures/rgBar.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0}
			},
			{
				-- Generic Element Configuration
				id="barHealth", type="Bar", parent="frame", layer=15, alpha=.8,
				attach = {
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=4, offsetY=4 },
					{ point="BOTTOMRIGHT", element="barResource", targetPoint="TOPRIGHT", offsetY=.5 },
				},
				-- visibilityBinding="id",
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="healthPercent", height=raidFrameTopBarHeight, colorBinding="callingColor",
				texAddon=AddonId, texFile="img/textures/rgBar.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				id="barAbsorb", type="Bar", parent="frame", layer=20,
				attach = {
					{ point="TOPLEFT", element="barResource", targetPoint="TOPLEFT", offsetX=0, offsetY=-14 },
					{ point="BOTTOMRIGHT", element="barResource", targetPoint="TOPRIGHT" }
				},
				growthDirection="right", 
				binding="absorbPercent",
				texAddon=AddonId, texFile="img/textures/rgBar.png", alpha=.8,
				color={r=0.08, g=0.91, b=1, a=0.9},
				backgroundColor={r=0.08, g=0.91, b=1, a=0}
			},
			{
				-- Generic Element Configuration
				id="tfHealthLabel", type="Label", parent="frame", layer=80, alpha=1,
				attach = {{ point="BOTTOMRIGHT", element="barHealth", targetPoint="BOTTOMRIGHT", offsetX=-4, offsetY=2 }},
				visibilityBinding="health",
				-- Type Specific Element Configuration
				text="{health}  /  {healthPercent}%", default="", color={r=1,g=1,b=1,a=1}, fontSize=12
			},
			{
				-- Generic Element Configuration
				id="tfAbsorbLabel", type="Label", parent="frame", layer=80,
				attach = {{ point="CENTERLEFT", element="barAbsorb", targetPoint="CENTERLEFT", offsetX=4, offsetY=1 }},
				visibilityBinding="id", 
				-- Type Specific Element Configuration
				text="{absorb}", default="", color={r=1, g=1, b=1, a=1}, fontSize=12,
			},
			{
				id="imgRole", type="MediaSet", parent="frame", layer=80, alpha=.8,
				attach = {{ point="CENTER", element="frameBorder", targetPoint="BOTTOMLEFT", offsetX=3, offsetY=-3 }},
				width = 26, height = 26,
				visibilityBinding="role",
				nameBinding="role", 
				names = { ["tank"] = "Tank", ["heal"] = "Heal", ["dps"] = "DPS", ["support"] = "Support" },
			},
			{
				-- Generic Element Configuration
				id="labelTest", type="Label", parent="frame", layer=80, alpha=1,
				attach = {{ point="BOTTOMCENTER", element="frameBorder", targetPoint="TOPCENTER", offsetX=0, offsetY=-6 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{guild}", default="", color={r=1,g=1,b=1,a=1}, fontSize=13,
			},
			{
				-- Generic Element Configuration
				id="labelName", type="Label", parent="frame", layer=80, alpha=1,
				attach = {{ point="TOPLEFT", element="barHealth", targetPoint="TOPLEFT", offsetX=4, offsetY=0 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{nameMed}", default="", color={r=1,g=1,b=1,a=1}, font="Bazooka", fontSize=14, outline=true
			},
			{
				-- Generic Element Configuration
				id="labelresourceR", type="Label", parent="frame", layer=80, alpha=1,
				attach = {{ point="CENTERRIGHT", element="barResource", targetPoint="CENTERRIGHT", offsetX=-4, offsetY=0 }},
				visibilityBinding="resource",
				-- Type Specific Element Configuration
				text="{resourcePercent}%", default="", color={r=1,g=1,b=1,a=1}, fontSize=10
			},
			{
				id="imgMark", type="MediaSet", parent="frame", layer=80,
				attach = {{ point="CENTERRIGHT", element="frameBorder", targetPoint="CENTERLEFT", offsetX=-4, offsetY=0 }},
				width = 26, height = 26,
				nameBinding="mark",
				names = 
				{
						["1"] = "riftMark01",
						["2"] = "riftMark02",
						["3"] = "riftMark03",
						["4"] = "riftMark04",
						["5"] = "riftMark05",
						["6"] = "riftMark06",
						["7"] = "riftMark07",
						["8"] = "riftMark08",
						["9"] = "riftMark09",
						["10"] = "riftMark10",
						["11"] = "riftMark11",
						["12"] = "riftMark12",
						["13"] = "riftMark13",
						["14"] = "riftMark14",
						["15"] = "riftMark15",
						["16"] = "riftMark16",
						["17"] = "riftMark17",
				},
				visibilityBinding="mark",alpha=1.0,
			},
			{
				id="imgPVP", type="MediaSet", parent="frame", layer=100, width=26, height=26,
			attach = {{ point="CENTER", element="frameBorder", targetPoint="TOPRIGHT", offsetX=-3, offsetY=3 }},
				nameBinding="pvpFaction",
				names = 
				{
					["defiant"] = "FactionDefiant",
					["guardian"] = "FactionGuardian",
				},
			},
			{
				id="imgConquest", type="MediaSet", parent="frame", layer=100, width=26, height=26,
			attach = {{ point="CENTER", element="frameBorder", targetPoint="BOTTOMRIGHT", offsetX=-3, offsetY=-3 }},
				nameBinding="cqFaction",
				names = 
				{
					["nightfall"] = "factionNightfall",
					["oathsworn"] = "factionOathsworn",
					["dominion"] = "factionDominion",
				},
			},
			{
				-- Generic Element Configuration
				id="labelDetail", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="TOPCENTER", element="frame", targetPoint="BOTTOMCENTER", offsetX=0, offsetY=4 }},
				visibilityBinding="raceName",
				-- Type Specific Element Configuration
				text="{raceName}", default="", color={r=1,g=1,b=1,a=1}, font="Bazooka", fontSize=14, outline=true
			},
			{
				-- Generic Element Configuration
				id="labelNPCDetail", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="TOPCENTER", element="frame", targetPoint="BOTTOMCENTER", offsetX=0, offsetY=4 }},
				visibilityBinding="tag",
				-- Type Specific Element Configuration
				text="{tagName}", default="", color={r=1,g=1,b=1,a=1}, font="Bazooka", fontSize=14, outline=true
			},
			{
				id="imgRare", type="Image", parent="frame", layer=60,
				attach = {{ point="CENTER", element="frameBorder", targetPoint="TOPCENTER", offsetX=0, offsetY=0 }},
				texAddon=AddonId, texFile="img/icons/blueDiamond.png",
				visibilityBinding="guaranteedLoot",
				width=40, height=40,
			},	
			{
				id="imgLevel", type="MediaSet", parent="frame", layer=60, alpha=1,
				attach = {{ point="CENTER", element="frameBorder", targetPoint="TOPLEFT", offsetX=4, offsetY=4 }},
				width=36, height=36,
				visibilityBinding="levelBorder",
				nameBinding="levelBorder",
				names =
				{
						["eliteIcon"] = "eliteIcon",
						["normalIcon"] = "normalIcon",
				},
			},
			{
				id="imgLevelBoss", type="Image", parent="frame", layer=70,
				attach = {{ point="CENTER", element="imgLevel", targetPoint="CENTER", offsetX=0, offsetY=0 }},
				texAddon=AddonId, texFile="img/icons/bossIcon.png", alpha=1,
				visibilityBinding="idBoss",
				width=54, height=54,
			},
			{
				id="levelLabel", type="Label", parent="frame", layer=80,
				attach = {{ point="CENTER", element="imgLevel", targetPoint="CENTER", offsetX=0, offsetY=0 }},
				visibilityBinding="levelText",
				-- Type Specific Element Configuration
				text="{level}", default="", colorBinding="levelColor", font="Bazooka", fontSize=12
			},				
			{
				id="imgLevelUnknown", type="Image", parent="frame", layer=100,
				attach = {{ point="CENTER", element="imgLevel", targetPoint="CENTER", offsetX=0, offsetY=-2 }},
				texAddon=AddonId, texFile="img/icons/levelUnknown.png",
				visibilityBinding="levelUnknown",
				width=32, height=32,
			},
			{
				id="barCast", type="Bar", parent="frame", layer=30,
				attach = {
					{ point="TOPLEFT", element="barResource", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="barResource", targetPoint="BOTTOMRIGHT" },
				},
				visibilityBinding="castName",
				binding="castPercent",
				texAddon=AddonId, texFile="img/textures/castbarCyan.png", colorBinding="castColor",
				backgroundColor={r=0, g=0, b=0, a=0.7}
			},
			{
				id="labelCast", type="Label", parent="frame", layer=40,
				attach = {{ point="CENTERLEFT", element="barCast", targetPoint="CENTERLEFT", offsetX=6, offsetY=0 }},
				visibilityBinding="castName",
				text="{castName}", default="", color={r=1,g=1,b=1,a=1}, fontSize=12
			},
			{
				id="buffPanelDebuffs", type="BuffPanel", parent="frame", layer=90,
				attach = {{ point="BOTTOMRIGHT", element="frameBorder", targetPoint="TOPRIGHT", offsetX=-4, offsetY=-34 }},
				rows=2, cols=4, iconSize=32, iconSpacingHorizontal=1, iconSpacingVertical=14, borderThickness=1, 
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=false, acceptHighPriorityBuffs=false, acceptCriticalPriorityBuffs=false,
				acceptLowPriorityDebuffs=true, acceptMediumPriorityDebuffs=true, acceptHighPriorityDebuffs=true, acceptCriticalPriorityDebuffs=true,
				growthDirection = "left_up", borderColor={r=1,g=1,b=1,a=1},
				timerSize=14, timerOffsetX=0, timerOffsetY=26,
				stackSize=16, stackOffsetX=0, stackOffsetY=0, stackBackgroundColor={r=0,g=0,b=0,a=0.3},
				sweepOverlay=false,
			},
			{
				id="buffPanelBuffs", type="BuffPanel", parent="HorizontalBar", layer=90,
				attach = {{ point="TOPLEFT", element="frameBorder", targetPoint="BOTTOMLEFT", offsetX=3, offsetY=14 }},
				rows=2, cols=5, iconSize=27, iconSpacingHorizontal=1, iconSpacingVertical=14, borderThickness=1,
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=true, acceptHighPriorityBuffs=true, acceptCriticalPriorityBuffs=true,
				acceptLowPriorityDebuffs=false, acceptMediumPriorityDebuffs=false, acceptHighPriorityDebuffs=false, acceptCriticalPriorityDebuffs=false,
				growthDirection = "right_down", selfCast=false,
				timerSize=10, timerOffsetX=0, timerOffsetY=20,
				stackSize=16, stackOffsetX=0, stackOffsetY=0, stackBackgroundColor={r=0,g=0,b=0,a=0.3},
				borderColor={r=0,g=0,b=0,a=1},
				sweepOverlay=false,
			},
			
		}
	}
	
	if WT.UnitFrame.EnableResizableTemplate then
            WT.UnitFrame.EnableResizableTemplate(self, rgTargetFrameWidth, rgTargetFrameHeight, template.elements)
    end
	
	for idx,element in ipairs(template.elements) do
		local showElement = true
		if options.excludeBuffs and element.type=="BuffPanel" then showElement = false end
		if options.excludeCasts and ((element.id == "barCast") or (element.id == "labelCast")) then showElement = false end
		if not options.showAbsorb and (element.id=="barAbsorb" or element.id=="labelHealthAndAbsorbPercent") then 
			showElement = false 
		end
		if showElement then
			self:CreateElement(element)
		end 
	end
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
end

------------------------------------------------------------------------------------------------------------
-- Local config options 
------------------------------------------------------------------------------------------------------------
local rgTTFrameWidth = 160
local rgTTFrameHeight = 40

------------------------------------------------------------------------------------------------------------
-- Frame Configuration Options
------------------------------------------------------------------------------------------------------------
local rgTTFrame = WT.UnitFrame:Template("rgTTFrame")
rgTTFrame.Configuration.Name = "Realm Guardians Tgt of Tgt Frame"
rgTTFrame.Configuration.RaidSuitable = false
rgTTFrame.Configuration.FrameType = "Frame"
rgTTFrame.Configuration.Width = rgTTFrameWidth
rgTTFrame.Configuration.Height = rgTTFrameHeight

-- Only enable resizing if the installed version of Gadgets can handle it:
if WT.UnitFrame.EnableResizableTemplate then
      rgTTFrame.Configuration.Resizable = { rgTTFrameWidth * 0.75, rgTTFrameHeight * 0.75, rgTTFrameWidth * 2.0, rgTTFrameHeight * 2.0 }
end

------------------------------------------------------------------------------------------------------------
-- Override the buff filter to hide some buffs
------------------------------------------------------------------------------------------------------------
local buffPriorities = 
{
		["Touched by Greed"] = 0,
	["Track Wood"] = 0,
	["Track Ore"] = 0,
	["Track Plants"] = 0,
	["Track Fish"] = 0,
	["Rested"] = 0,
	["Prismatic Glory"] = 0,
	["Looking for Group Cooldown"] = 0,
	["Feast of Domination"] = 0,
	["Feast of Vengeance"] = 0,
	["Poisonous Coating"] = 0,
	["Remarkable Whetstone"] = 0,
	["Luminescent Powerstone"] = 0,
	["Comfortable Riding Chaps"] = 0,
	["Comfortable Insoles"] = 0,
	["Quantum Sight"] = 0,
	["Planar Protection"] = 0,
	["Stellar Brightsurge Vial"] = 0,
	["Stellar Powersurge Vial"] = 0,
	["Stellar Enduring Vial"] = 0,
	["Stellar Fortified Vial"] = 0,
	["Summon Banner of Rage"] = 0,
	["Summon Banner of Concentration"] = 0,
	["Summon Banner of Fortitude"] = 0,
	["Summon Banner of Protection"] = 0,
	["Fanfare of Vigor"] = 0,
	["Fanfare of Power"] = 0,
	["Ascended Courage"] = 0,
	["Planar Protection"] = 0,
	["Heroic Resolve"] = 0,
	["Empowerment: Mounted Swiftness"] = 0,
	["Empowerment: Offense"] = 0,
	["Empowerment: Healing"] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
}
function rgTTFrame:GetBuffPriority(buff)
	if not buff then return 2 end
	return buffPriorities[buff.name] or 2
end

------------------------------------------------------------------------------------------------------------
--
------------------------------------------------------------------------------------------------------------
function rgTTFrame:Construct(options)

	local template =
	{
		elements = 
		{
			{
				-- Generic Element Configuration
				id="frameBackdrop", type="Frame", parent="frame", layer=0,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=-6, offsetY=-10 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=6, offsetY=10 },
				},
				visibilityBinding="id", 
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=.5,
				color={r=0,g=0,b=0,a=0},
			},
			{
				-- Generic Element Configuration
				id="frameBorder", type="MediaSet", parent="frame", layer=4, alpha=1,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=-8, offsetY=-7.25 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=5.5, offsetY=7.25 },
				},
				visibilityBinding="idEliteTT", 
				nameBinding="idEliteTT",
				names = 
				{
					["eliteBorder2"] = "eliteBorder2",
					["normalBorder2"] = "normalBorder2",
				},
			},
			{
				-- Generic Element Configuration
				id="bossBorder", type="Image", parent="frame", layer=2, alpha=1,
				attach = { 
					{ point="TOPLEFT", element="frameBorder", targetPoint="TOPLEFT", offsetX=-5, offsetY=-6 },
					{ point="BOTTOMRIGHT", element="frameBorder", targetPoint="BOTTOMRIGHT", offsetX=4, offsetY=6},
				},
				visibilityBinding="idBoss", 
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/borders/bossBorder.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="rareBorder", type="Image", parent="frame", layer=2, alpha=1,
				attach = { 
					{ point="TOPLEFT", element="frameBorder", targetPoint="TOPLEFT", offsetX=0, offsetY=0 },
					{ point="BOTTOMRIGHT", element="frameBorder", targetPoint="BOTTOMRIGHT", offsetX=0, offsetY=0 },
				},
				visibilityBinding="guaranteedLoot", 
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/borders/rareBorder.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},	
			{
				-- Generic Element Configuration
				id="frameBlocked", type="Frame", parent="frame", layer=100, visibilityBinding="blockedOrOutOfRange",
				color={r=0,g=0,b=0},alpha=0.6,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=0, offsetY=0 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=0, offsetY=0 } 
				},
			},
			{
				-- Generic Element Configuration
				id="barHealth", type="Bar", parent="frame", layer=20, alpha=.8,
				attach = {
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=2, offsetY=2 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-2, offsetY=-1.75 },
				},
				-- visibilityBinding="id",
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="healthPercent", colorBinding="callingColor",
				texAddon=AddonId, texFile="img/textures/rgBar.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="barCleanse", type="Bar", parent="frame", layer=25, alpha=1,
				attach = {
					{ point="TOPLEFT", element="barHealth", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="barHealth", targetPoint="BOTTOMRIGHT" },
				},
				visibilityBinding="cleanseBar",
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="healthPercent", color={r=0, g=0, b=0, a=0},
				texAddon=AddonId, texFile="img/textures/rgBarCleanse.png", alpha=1,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				id="barAbsorb", type="Bar", parent="frame", layer=30,
				attach = {
					{ point="TOPLEFT", element="frame", targetPoint="BOTTOMLEFT", offsetX=0, offsetY=-14 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" }
				},
				growthDirection="right", 
				binding="absorbPercent",
				color={r=0.08, g=0.91, b=1, a=0.9},
				texAddon=AddonId, texFile="img/textures/rgBar.png", alpha=.8,
				backgroundColor={r=0.08, g=0.91, b=1, a=0}
			},
			{
				-- Generic Element Configuration
				id="labelHealthAndAbsorbPercent", type="Label", parent="frame", layer=80,
				attach = {{ point="CENTERLEFT", element="barAbsorb", targetPoint="CENTERLEFT", offsetX=4, offsetY=-2 }},
				visibilityBinding="id", 
				-- Type Specific Element Configuration
				text="{absorb}", default="", colorBinding="cleanseTextColor", fontSize=12,
			},
			{
				-- Generic Element Configuration
				id="labelHealth", type="Label", parent="frame", layer=80, alpha=1,
				attach = {{ point="BOTTOMRIGHT", element="barHealth", targetPoint="BOTTOMRIGHT", offsetX=-2, offsetY=4}},
				visibilityBinding="health",
				-- Type Specific Element Configuration
				text="{health} / {healthPercent}%", default="", colorBinding="cleanseTextColor", fontSize=12
			},
			{
				-- Generic Element Configuration
				id="labelName", type="Label", parent="frame", layer=80, alpha=1,
				attach = {{ point="TOPLEFT", element="barHealth", targetPoint="TOPLEFT", offsetX=4, offsetY=2 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{nameMini}", default="", colorBinding="cleanseTextColor", font="Bazooka", fontSize=14, 
			},
			{
				id="imgMark", type="MediaSet", parent="frame", layer=60,
				attach = {{ point="CENTERLEFT", element="frameBorder", targetPoint="CENTERRIGHT", offsetX=4, offsetY=0 }},
				width = 24, height = 24,
				nameBinding="mark",
				names = 
					{
						["1"] = "riftMark01",
						["2"] = "riftMark02",
						["3"] = "riftMark03",
						["4"] = "riftMark04",
						["5"] = "riftMark05",
						["6"] = "riftMark06",
						["7"] = "riftMark07",
						["8"] = "riftMark08",
						["9"] = "riftMark09",
						["10"] = "riftMark10",
						["11"] = "riftMark11",
						["12"] = "riftMark12",
						["13"] = "riftMark13",
						["14"] = "riftMark14",
						["15"] = "riftMark15",
						["16"] = "riftMark16",
						["17"] = "riftMark17",
					},
				visibilityBinding="mark", alpha=1.0,
			},
			{
				id="imgLevel", type="MediaSet", parent="frame", layer=60, alpha=1,
				attach = {{ point="CENTER", element="frameBorder", targetPoint="TOPLEFT", offsetX=4, offsetY=4 }},
				width=36, height=36,
				visibilityBinding="levelBorder",
				nameBinding="levelBorder",
				names =
				{
						["eliteIcon"] = "eliteIcon",
						["normalIcon"] = "normalIcon",
				},
			},
			{
				id="imgLevelBoss", type="Image", parent="frame", layer=70,
				attach = {{ point="CENTER", element="imgLevel", targetPoint="CENTER", offsetX=0, offsetY=0 }},
				texAddon=AddonId, texFile="img/icons/bossIcon.png", alpha=1,
				visibilityBinding="idBoss",
				width=46, height=46,
			},
			{
				id="levelLabel", type="Label", parent="frame", layer=80,
				attach = {{ point="CENTER", element="imgLevel", targetPoint="CENTER", offsetX=0, offsetY=0 }},
				visibilityBinding="levelText",
				-- Type Specific Element Configuration
				text="{level}", default="", colorBinding="levelColor", font="Bazooka", fontSize=12
			},				
			{
				id="imgLevelUnknown", type="Image", parent="frame", layer=100,
				attach = {{ point="CENTER", element="imgLevel", targetPoint="CENTER", offsetX=0, offsetY=-2 }},
				texAddon=AddonId, texFile="img/icons/levelUnknown.png",
				visibilityBinding="levelUnknown",
				width=32, height=32,
			},
			{
				id="imgRare", type="Image", parent="frame", layer=60,
				attach = {{ point="CENTER", element="frameBorder", targetPoint="TOPCENTER", offsetX=0, offsetY=0 }},
				texAddon=AddonId, texFile="img/icons/blueDiamond.png",
				visibilityBinding="guaranteedLoot",
				width=32, height=32,
			},	
			{
				id="buffPanelDebuffs", type="BuffPanel", parent="frame", layer=90,
				attach = {{ point="BOTTOMRIGHT", element="frameBorder", targetPoint="TOPRIGHT", offsetX=-4, offsetY=-24 }},
				rows=2, cols=4, iconSize=32, iconSpacingHorizontal=1, iconSpacingVertical=14, borderThickness=1, 
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=false, acceptHighPriorityBuffs=false, acceptCriticalPriorityBuffs=false,
				acceptLowPriorityDebuffs=true, acceptMediumPriorityDebuffs=true, acceptHighPriorityDebuffs=true, acceptCriticalPriorityDebuffs=true,
				growthDirection = "left_up", borderColor={r=1,g=1,b=1,a=1},
				timerSize=12, timerOffsetX=0, timerOffsetY=22, timerBackgroundColor={r=0,g=0,b=0,a=0.2},
				stackSize=12, stackOffsetX=0, stackOffsetY=0, stackBackgroundColor={r=0,g=0,b=0,a=0.3},
				sweepOverlay=false,
			},
			{
				id="buffPanelBuffs", type="BuffPanel", parent="HorizontalBar", layer=90,
				attach = {{ point="TOPLEFT", element="frameBorder", targetPoint="BOTTOMLEFT", offsetX=0, offsetY=6 }},
				rows=2, cols=5, iconSize=25, iconSpacingHorizontal=1, iconSpacingVertical=14, borderThickness=1,
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=true, acceptHighPriorityBuffs=true, acceptCriticalPriorityBuffs=true,
				acceptLowPriorityDebuffs=false, acceptMediumPriorityDebuffs=false, acceptHighPriorityDebuffs=false, acceptCriticalPriorityDebuffs=false,
				growthDirection = "right_down", selfCast=false,
				timerSize=10, timerOffsetX=0, timerOffsetY=20, timerBackgroundColor={r=0,g=0,b=0,a=0.2},
				stackSize=20, stackOffsetX=0, stackOffsetY=0, stackBackgroundColor={r=0,g=0,b=0,a=0.3},
				borderColor={r=0,g=0,b=0,a=1},
				sweepOverlay=false,
			},
		}
	}
	
	if WT.UnitFrame.EnableResizableTemplate then
            WT.UnitFrame.EnableResizableTemplate(self, rgTTFrameWidth, rgTTFrameHeight, template.elements)
    end
	
	for idx,element in ipairs(template.elements) do
		local showElement = true
		if options.excludeBuffs and element.type=="BuffPanel" then showElement = false end
		if options.excludeCasts and ((element.id == "barCast") or (element.id == "labelCast")) then showElement = false end
		if not options.showAbsorb and (element.id=="barAbsorb" or element.id=="labelHealthAndAbsorbPercent") then 
			showElement = false 
		end
		if showElement then
			self:CreateElement(element)
		end 
	end
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
end

