local toc, data = ...
local AddonId = toc.identifier

------------------------------------------------------------------------------------------------------------
-- Local config options 
------------------------------------------------------------------------------------------------------------
local rgTargetFrameWidth = 190
local rgTargetFrameHeight = 80
------------------------------------------------------------------------------------------------------------
-- Frame Configuration Options 
------------------------------------------------------------------------------------------------------------
local rgTargetFrame = WT.UnitFrame:Template("rgTargetFrame")
rgTargetFrame.Configuration.Name = "Realm Guardians Target Frame"
rgTargetFrame.Configuration.RaidSuitable = False
rgTargetFrame.Configuration.FrameType = "Frame"
rgTargetFrame.Configuration.Width = rgTargetFrameWidth 
rgTargetFrame.Configuration.Height = rgTargetFrameHeight

if WT.UnitFrame.EnableResizableTemplate then
      rgTargetFrame.Configuration.Resizable = { rgTargetFrameWidth * 0.50, rgTargetFrameHeight * 0.50, rgTargetFrameWidth * 1.5, rgTargetFrameHeight * 1.5 }
end

------------------------------------------------------------------------------------------------------------
-- Override the buff filter to hide some buffs
------------------------------------------------------------------------------------------------------------
local buffPriorities = 
{
	["Touched by Greed"] = 0,
	["Track Wood"] = 0,
	["Track Ore"] = 0,
	["Track Plants"] = 0,
	["Track Fish"] = 0,
	["Rested"] = 0,
	["Prismatic Glory"] = 0,
	["Looking for Group Cooldown"] = 0,
	["Feast of Domination"] = 0,
	["Feast of Vengeance"] = 0,
	["Poisonous Coating"] = 0,
	["Remarkable Whetstone"] = 0,
	["Luminescent Powerstone"] = 0,
	["Comfortable Riding Chaps"] = 0,
	["Comfortable Insoles"] = 0,
	["Quantum Sight"] = 0,
	["Planar Protection"] = 0,
	["Stellar Brightsurge Vial"] = 0,
	["Stellar Powersurge Vial"] = 0,
	["Stellar Enduring Vial"] = 0,
	["Stellar Fortified Vial"] = 0,
	["Summon Banner of Rage"] = 0,
	["Summon Banner of Concentration"] = 0,
	["Summon Banner of Fortitude"] = 0,
	["Summon Banner of Protection"] = 0,
	["Fanfare of Vigor"] = 0,
	["Fanfare of Power"] = 0,
	["Ascended Courage"] = 0,
	["Planar Protection"] = 0,
	["Heroic Resolve"] = 0,
	["Empowerment: Mounted Swiftness"] = 0,
	["Empowerment: Offense"] = 0,
	["Empowerment: Healing"] = 0,
	["Thick Armor Plating"] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	
}
function rgTargetFrame:GetBuffPriority(buff)
	if not buff then return 2 end
	return buffPriorities[buff.name] or 2
end

------------------------------------------------------------------------------------------------------------
--
------------------------------------------------------------------------------------------------------------
function rgTargetFrame:Construct(options)

	local template =
	{
		elements = 
		{
			{
				id="frameBackdrop", type="MediaSet", parent="frame", layer=0, alpha=.9,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" }, 
				},
				visibilityBinding="targetBackdropIdElite", 
				nameBinding="targetBackdropIdElite",
				names = 
				{
					["targetEliteBackdrop"] = "targetEliteBackdrop",
					["targetEliteBackdrop_tagged"] = "targetEliteBackdrop_tagged",
					["targetBackdrop"] = "targetBackdrop",
					["targetBackdrop_tagged"] = "targetBackdrop_tagged",
				},
			},
			{
				id="innerFrame", type="MediaSet", parent="frame", layer=2, alpha=.9,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=10, offsetY=18 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-10, offsetY=-18 }, 
				},
				visibilityBinding="targetInnerIdElite", 
				nameBinding="targetInnerIdElite",
				names = 
				{
					["targetEliteInnerFrame"] = "targetEliteInnerFrame",
					["targetInnerFrame"] = "targetInnerFrame",
				},
			},
			{
				id="bossBorder", type="Image", parent="frame", layer=100, alpha=1,
				attach = 
				{ 
					{ point="TOPLEFT", element="frameBackdrop", targetPoint="TOPLEFT", offsetX=-3, offsetY=-2 },
					{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT", offsetX=2.75, offsetY=2},
				},
				visibilityBinding="idBoss", 
				texAddon=AddonId, texFile="img/borders/targetBossBorder.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			--[[
			{
				id="rareBorder", type="Image", parent="frame", layer=4, alpha=1,
				attach = 
				{ 
					{ point="TOPLEFT", element="frameBackdrop", targetPoint="TOPLEFT", offsetX=-4, offsetY=-4 },
					{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT", offsetX=4, offsetY=4},
				},
				visibilityBinding="guaranteedLoot", 
				texAddon=AddonId, texFile="img/borders/targetRareBorder.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			--]]			
			{
				id="barResource", type="Bar", parent="frame", layer=15, alpha=.8,
				attach = 
				{
					{ point="TOPLEFT", element="innerFrame", targetPoint={.025, .55} },
					{ point="BOTTOMRIGHT", element="innerFrame", targetPoint="BOTTOMRIGHT", offsetX=-5, offsetY=-4.5 }
				},
				visibilityBinding="id",
				growthDirection="right",
				binding="resourcePercent", colorBinding="resourceColor",
				texAddon=AddonId, texFile="img/textures/rgBlackBar.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0}
			},
			{
				id="barHealth", type="Bar", parent="frame", layer=15, alpha=.8,
				attach = 
				{
					{ point="TOPLEFT", element="innerFrame", targetPoint="TOPLEFT", offsetX=4, offsetY=4.25 },
					{ point="BOTTOMRIGHT", element="innerFrame", targetPoint={.970, .422} },
				},
				visibilityBinding="id",
				growthDirection="right",
				binding="healthPercent", colorBinding="callingColor",
				texAddon=AddonId, texFile="img/textures/rgBlackBar.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				id="barAbsorb", type="Bar", parent="frame", layer=20,
				attach = 
				{
					{ point="TOPLEFT", element="barHealth", targetPoint={.005, 1/6} },
					{ point="BOTTOMRIGHT", element="barHealth", targetPoint={.995, 5/6} },
				},
				growthDirection="right", 
				binding="absorbPercent",
				--texAddon=AddonId, texFile="img/textures/rgBlackBar.png", alpha=.5,
				color={r=0.08, g=0.91, b=1, a=0.9},
				backgroundColor={r=0.08, g=0.91, b=1, a=0}
			},
			{
				id="imgRole", type="MediaSet", parent="frame", layer=180, alpha=1,
				attach = {{ point="CENTER", element="frameBackdrop", targetPoint="BOTTOMLEFT", offsetX=6, offsetY=-5 }},
				width = 18, height = 18,
				visibilityBinding="role",
				nameBinding="role", 
				names = { ["tank"] = "rgRaidTank", ["heal"] = "rgRaidHeal", ["dps"] = "rgRaidDPS", ["support"] = "rgRaidSupport" },
			},
			{
				id="imgMark", type="MediaSet", parent="frame", layer=180,
				attach = {{ point="CENTER", element="frameBackdrop", targetPoint="TOPRIGHT", offsetX=-6, offsetY=5 }},
				width = 28, height = 28,
				nameBinding="mark",
				names = 
				{
						["1"] = "riftMark01",
						["2"] = "riftMark02",
						["3"] = "riftMark03",
						["4"] = "riftMark04",
						["5"] = "riftMark05",
						["6"] = "riftMark06",
						["7"] = "riftMark07",
						["8"] = "riftMark08",
						["9"] = "riftMark09",
						["10"] = "riftMark10",
						["11"] = "riftMark11",
						["12"] = "riftMark12",
						["13"] = "riftMark13",
						["14"] = "riftMark14",
						["15"] = "riftMark15",
						["16"] = "riftMark16",
						["17"] = "riftMark17",
				},
				visibilityBinding="mark",alpha=1.0,
			},
			{
				id="imgPVP", type="MediaSet", parent="frame", layer=100, width=22, height=22,
			attach = {{ point="CENTER", element="frameBackdrop", targetPoint="BOTTOMRIGHT", offsetX=-6, offsetY=-5 }},
				nameBinding="pvpFaction",
				names = 
				{
					["defiant"] = "FactionDefiant",
					["guardian"] = "FactionGuardian",
				},
			},
			{
				id="imgConquest", type="MediaSet", parent="frame", layer=100, width=26, height=26,
			attach = {{ point="CENTERRIGHT", element="imgPVP", targetPoint="CENTERLEFT", offsetX=2, offsetY=0 }},
				nameBinding="cqFaction",
				names = 
				{
					["nightfall"] = "factionNightfall",
					["oathsworn"] = "factionOathsworn",
					["dominion"] = "factionDominion",
				},
			},
			{
				id="labelNameSecondary", type="Label", parent="frame", layer=200, alpha=1,
				attach = {{ point="BOTTOMCENTER", element="innerFrame", targetPoint="TOPCENTER", offsetX=2, offsetY=1.5 }},
				visibilityBinding="subName",
				text="< {subName} >", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler", fontSize=10, outline=true,
			},
			{
				id="labelName", type="Label", parent="frame", layer=200, alpha=1,
				attach = {{ point="BOTTOMCENTER", element="labelNameSecondary", targetPoint="TOPCENTER", offsetX=0, offsetY=4 }},
				visibilityBinding="name",
				text="{nameMed}", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler", fontSize=13, outline=true,
			},
			{
				id="labelRange", type="Label", parent="frame", layer=200, alpha=1,
				attach = {{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT", offsetX=-22, offsetY=-1.5 }},
				visibilityBinding="rangeDisplay",
				text="{range} m", default="", colorBinding="rangeColor", font="ArmWrestler", fontSize=12, 
			},
			{
				id="tfHealthLabel", type="Label", parent="frame", layer=200, alpha=1,
				attach = {{ point="CENTERRIGHT", element="barHealth", targetPoint="CENTERRIGHT", offsetX=-8, offsetY=0 }},
				visibilityBinding="healthText",
				text="{health}  ({healthPercent}%)", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler", fontSize=10, 
			},
			{
				id="tfAbsorbLabel", type="Label", parent="frame", layer=200,
				attach = {{ point="CENTERLEFT", element="barAbsorb", targetPoint="CENTERLEFT", offsetX=4, offsetY=0 }},
				visibilityBinding="id", 
				text="( {absorb} )", default="", color={r=1, g=1, b=1, a=1}, font="ArmWrestler", fontSize=9, outline=true,
			},
			{
				id="tfResourceLabel", type="Label", parent="frame", layer=200, alpha=1,
				attach = {{ point="CENTERRIGHT", element="barResource", targetPoint="CENTERRIGHT", offsetX=-8, offsetY=0 }},
				visibilityBinding="resourceText",
				text="{resource}  ({resourcePercent}%)", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler", fontSize=10, 
			},
			{
				id="tfStatusLabel", type="Label", parent="frame", layer=200,
				attach = {{ point="CENTER", element="barHealth", targetPoint="CENTER", offsetX=0, offsetY=1 }},
				visibilityBinding="rgStatus",
				text="{rgStatus}", default="", font="Bazooka", fontSize=12
			},
			{
				id="labelDetail", type="Label", parent="frame", layer=200, alpha=1,
				attach = {{ point="BOTTOMLEFT", element="frame", targetPoint="BOTTOMLEFT", offsetX=18, offsetY=-3 }},
				visibilityBinding="raceName",
				text="{raceName}", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler", fontSize=11, outline=true
			},
			{
				id="labelNPCDetail", type="Label", parent="frame", layer=200, alpha=1,
				attach = {{ point="BOTTOMLEFT", element="frame", targetPoint="BOTTOMLEFT", offsetX=14, offsetY=-3 }},
				visibilityBinding="tag",
				text="{tagName}", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler", fontSize=11, outline=true
			},
			{
				id="imgRare", type="Image", parent="frame", layer=60,
				attach = {{ point="CENTER", element="frameBackdrop", targetPoint="TOPCENTER", offsetX=0, offsetY=0 }},
				texAddon=AddonId, texFile="img/icons/blueDiamond.png",
				visibilityBinding="guaranteedLoot",
				width=40, height=40,
			},	
			{
				id="imgLevel", type="MediaSet", parent="frame", layer=130, alpha=1,
				attach = {{ point="CENTER", element="frameBackdrop", targetPoint="TOPLEFT", offsetX=4, offsetY=4 }},
				width=38, height=38,
				visibilityBinding="levelBorder",
				nameBinding="levelBorder",
				names =
				{
						["eliteIcon"] = "eliteIcon",
						["normalIcon"] = "normalIcon",
				},
			},
			{
				id="imgLevelBoss", type="Image", parent="frame", layer=140,
				attach = {{ point="CENTER", element="imgLevel", targetPoint="CENTER", offsetX=0, offsetY=0 }},
				texAddon=AddonId, texFile="img/icons/bossIcon.png", alpha=1,
				visibilityBinding="idBoss",
				width=44, height=44,
			},
			{
				id="levelLabel", type="Label", parent="frame", layer=150,
				attach = {{ point="CENTER", element="imgLevel", targetPoint="CENTER", offsetX=0, offsetY=0 }},
				visibilityBinding="levelText",
				colorBinding="difficultyColor", 
				text="{level}", default="", font="ArmWrestler", fontSize=13
			},				
			{
				id="imgLevelUnknown", type="Image", parent="frame", layer=160,
				attach = {{ point="CENTER", element="imgLevel", targetPoint="CENTER", offsetX=0, offsetY=-2 }},
				texAddon=AddonId, texFile="img/icons/levelUnknown.png",
				visibilityBinding="levelUnknown",
				width=32, height=32,
			},
			{
				id="barCast", type="Bar", parent="frame", layer=90, alpha=1,
				attach = {
					{ point="TOPLEFT", element="barResource", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="barResource", targetPoint="BOTTOMRIGHT" },
				},
				visibilityBinding="castName",
				binding="castPercent",
				texAddon=AddonId, texFile="img/textures/gloss.png", alpha=.6,
				colorBinding="castColor",
				backgroundColor={r=0, g=0, b=0, a=1}
			},
			{
				id="labelCast", type="Label", parent="frame", layer=100,
				attach = {{ point="CENTER", element="barCast", targetPoint="CENTER", offsetX=0, offsetY=0 }},
				visibilityBinding="castName",
				text="{castName}", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler",  fontSize=11, outline=true,
			},
			{
				id="buffPanelDebuffs", type="BuffPanel", parent="frame", layer=90,
				attach = {{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="TOPRIGHT", offsetX=-4, offsetY=-28 }},
				rows=2, cols=4, iconSize=28, iconSpacingHorizontal=1, iconSpacingVertical=14, borderThickness=1, 
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=false, acceptHighPriorityBuffs=false, acceptCriticalPriorityBuffs=false,
				acceptLowPriorityDebuffs=true, acceptMediumPriorityDebuffs=true, acceptHighPriorityDebuffs=true, acceptCriticalPriorityDebuffs=true,
				growthDirection = "left_up", borderColor={r=1,g=1,b=1,a=1},
				timerSize=12, timerOffsetX=0, timerOffsetY=20,
				stackSize=14, stackOffsetX=0, stackOffsetY=0, stackBackgroundColor={r=0,g=0,b=0,a=0.3},
				sweepOverlay=false,
			},
			{
				id="buffPanelBuffs", type="BuffPanel", parent="HorizontalBar", layer=90,
				attach = {{ point="TOPLEFT", element="frameBackdrop", targetPoint="BOTTOMLEFT", offsetX=3, offsetY=8 }},
				rows=2, cols=5, iconSize=24, iconSpacingHorizontal=1, iconSpacingVertical=14, borderThickness=1,
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=true, acceptHighPriorityBuffs=true, acceptCriticalPriorityBuffs=true,
				acceptLowPriorityDebuffs=false, acceptMediumPriorityDebuffs=false, acceptHighPriorityDebuffs=false, acceptCriticalPriorityDebuffs=false,
				growthDirection = "right_down", selfCast=false,
				timerSize=11, timerOffsetX=0, timerOffsetY=21,
				stackSize=16, stackOffsetX=0, stackOffsetY=0, stackBackgroundColor={r=0,g=0,b=0,a=0.3},
				borderColor={r=0,g=0,b=0,a=1},
				sweepOverlay=false,
			},
			{
				id="frameBlocked", type="Frame", parent="frame", layer=120, visibilityBinding="blockedOrOutOfRange",
				color={r=0,g=0,b=0},alpha=0.6,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=-3, offsetY=-5 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=3, offsetY=5 }, 
				},
			}, 
		}
	}
	
	if WT.UnitFrame.EnableResizableTemplate then
            WT.UnitFrame.EnableResizableTemplate(self, rgTargetFrameWidth, rgTargetFrameHeight, template.elements)
    end
	
	for idx,element in ipairs(template.elements) do
		local showElement = true
		if options.excludeBuffs and element.type=="BuffPanel" then showElement = false end
		if options.excludeCasts and ((element.id == "barCast") or (element.id == "labelCast")) then showElement = false end
		if not options.showAbsorb and (element.id=="barAbsorb" or element.id=="labelHealthAndAbsorbPercent") then 
			showElement = false 
		end
		if showElement then
			self:CreateElement(element)
		end 
	end
	
	local elSecondary = self.Elements.labelNameSecondary
	local origFn = elSecondary.SetText
	function elSecondary:SetText(txt)
		origFn(elSecondary, txt)
		if txt == "" then
            elSecondary:SetHeight(0)
		else
            elSecondary:ClearHeight()
		end
	end
	
	--[[
	self:CreateBinding("calling", self, 		
		function(frame, newCalling)	
			local img = self.Elements.barResource.Image
			if not newCalling then return end
			if newCalling == "warrior" then img:SetTexture(AddonId, "img/textures/warriorNeon.png") end
			if newCalling == "rogue" then img:SetTexture(AddonId, "img/textures/rogueNeon.png") end
			if newCalling == "mage" then img:SetTexture(AddonId, "img/textures/mageNeon.png") end		
			if newCalling == "cleric" then img:SetTexture(AddonId, "img/textures/clericNeon.png") end	
	end	)
	--]]
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
end

------------------------------------------------------------------------------------------------------------
-- Local config options 
------------------------------------------------------------------------------------------------------------
local rgTTFrameWidth = 140
local rgTTFrameHeight = 50

------------------------------------------------------------------------------------------------------------
-- Frame Configuration Options
------------------------------------------------------------------------------------------------------------
local rgTTFrame = WT.UnitFrame:Template("rgTTFrame")
rgTTFrame.Configuration.Name = "Realm Guardians Tgt of Tgt Frame"
rgTTFrame.Configuration.RaidSuitable = false
rgTTFrame.Configuration.FrameType = "Frame"
rgTTFrame.Configuration.Width = rgTTFrameWidth
rgTTFrame.Configuration.Height = rgTTFrameHeight

-- Only enable resizing if the installed version of Gadgets can handle it:
if WT.UnitFrame.EnableResizableTemplate then
      rgTTFrame.Configuration.Resizable = { rgTTFrameWidth * 0.50, rgTTFrameHeight * 0.50, rgTTFrameWidth * 1.5, rgTTFrameHeight * 1.5 }
end

------------------------------------------------------------------------------------------------------------
-- Override the buff filter to hide some buffs
------------------------------------------------------------------------------------------------------------
local buffPriorities = 
{
		["Touched by Greed"] = 0,
	["Track Wood"] = 0,
	["Track Ore"] = 0,
	["Track Plants"] = 0,
	["Track Fish"] = 0,
	["Rested"] = 0,
	["Prismatic Glory"] = 0,
	["Looking for Group Cooldown"] = 0,
	["Feast of Domination"] = 0,
	["Feast of Vengeance"] = 0,
	["Poisonous Coating"] = 0,
	["Remarkable Whetstone"] = 0,
	["Luminescent Powerstone"] = 0,
	["Comfortable Riding Chaps"] = 0,
	["Comfortable Insoles"] = 0,
	["Quantum Sight"] = 0,
	["Planar Protection"] = 0,
	["Stellar Brightsurge Vial"] = 0,
	["Stellar Powersurge Vial"] = 0,
	["Stellar Enduring Vial"] = 0,
	["Stellar Fortified Vial"] = 0,
	["Summon Banner of Rage"] = 0,
	["Summon Banner of Concentration"] = 0,
	["Summon Banner of Fortitude"] = 0,
	["Summon Banner of Protection"] = 0,
	["Fanfare of Vigor"] = 0,
	["Fanfare of Power"] = 0,
	["Ascended Courage"] = 0,
	["Planar Protection"] = 0,
	["Heroic Resolve"] = 0,
	["Empowerment: Mounted Swiftness"] = 0,
	["Empowerment: Offense"] = 0,
	["Empowerment: Healing"] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
	[""] = 0,
}
function rgTTFrame:GetBuffPriority(buff)
	if not buff then return 2 end
	return buffPriorities[buff.name] or 2
end

------------------------------------------------------------------------------------------------------------
--
------------------------------------------------------------------------------------------------------------
function rgTTFrame:Construct(options)

	local template =
	{
		elements = 
		{
			{
				id="ttFrameBackdrop", type="MediaSet", parent="frame", layer=0, alpha=.8,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=10, offsetY=-2 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-10, offsetY=2 }, 
				},
				visibilityBinding="targetBackdropIdElite", 
				nameBinding="targetBackdropIdElite",
				names = 
				{
					["targetEliteBackdrop"] = "targetEliteBackdrop",
					["targetEliteBackdrop_tagged"] = "targetEliteBackdrop_tagged",
					["targetBackdrop"] = "targetBackdrop",
					["targetBackdrop_tagged"] = "targetBackdrop_tagged",
				},
			},
			{
				id="ttInnerFrame", type="MediaSet", parent="frame", layer=8, alpha=.9,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=-10, offsetY=14 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=10, offsetY=-14.75 }, 
				},
				visibilityBinding="innerIdEliteTT", 
				nameBinding="innerIdEliteTT",
				names = 
				{
					["eliteInnerFrameTT"] = "eliteInnerFrameTT",
					["innerFrameTT"] = "innerFrameTT",
				},
			},
			{
				id="ttBossBorderOuter", type="Image", parent="frame", layer=4, alpha=1,
				attach = 
				{ 
					{ point="TOPLEFT", element="ttFrameBackdrop", targetPoint="TOPLEFT", offsetX=0, offsetY=0 },
					{ point="BOTTOMRIGHT", element="ttFrameBackdrop", targetPoint="BOTTOMRIGHT", offsetX=0, offsetY=0 },
				},
				visibilityBinding="idBoss", 
				texAddon=AddonId, texFile="img/borders/ttBossBorder.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			--[[
			{
				id="ttBossBorderInner", type="Image", parent="frame", layer=90, alpha=1,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=-14, offsetY=10 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=14, offsetY=-10.75 },
				},
				visibilityBinding="idBoss", 
				texAddon=AddonId, texFile="img/borders/ttBossBorder.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				id="ttRareBorder", type="Image", parent="frame", layer=4, alpha=1,
				attach = 
				{ 
					{ point="TOPLEFT", element="ttFrameBackdrop", targetPoint="TOPLEFT", offsetX=-4, offsetY=-4 },
					{ point="BOTTOMRIGHT", element="ttFrameBackdrop", targetPoint="BOTTOMRIGHT", offsetX=4, offsetY=4},
				},
				visibilityBinding="guaranteedLoot", 
				texAddon=AddonId, texFile="img/borders/targetRareBorder.png", alpha=1, 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			--]]			
			{
				id="ttBarHealth", type="Bar", parent="frame", layer=20, alpha=.8,
				attach = 
				{
					{ point="TOPLEFT", element="ttInnerFrame", targetPoint="TOPLEFT", offsetX=4, offsetY=3 },
					{ point="BOTTOMRIGHT", element="ttInnerFrame", targetPoint="BOTTOMRIGHT", offsetX=-5, offsetY=-3 }, 
				},
				visibilityBinding="id",
				growthDirection="right",
				binding="healthPercent", colorBinding="callingColor",
				texAddon=AddonId, texFile="img/textures/rgBlackBar.png", alpha=.8,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				id="ttBarCleanse", type="Bar", parent="frame", layer=25, alpha=.6,
				attach = 
				{
					{ point="TOPLEFT", element="ttBarHealth", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="ttBarHealth", targetPoint="BOTTOMRIGHT" },
				},
				visibilityBinding="cleanseBar",
				growthDirection="right",
				binding="healthPercent", color={r=1, g=.41, b=.71, a=.6},
				--texAddon=AddonId, texFile="img/textures/rgBarCleanse.png", alpha=1,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				id="ttBarAbsorb", type="Bar", parent="frame", layer=30,
				attach = 
				{
					{ point="TOPLEFT", element="ttBarHealth", targetPoint={.005, 1/5} },
					{ point="BOTTOMRIGHT", element="ttBarHealth", targetPoint={.995, 4/5} },
				},
				growthDirection="right", 
				binding="absorbPercent",
				color={r=0.08, g=0.91, b=1, a=0.9},
				--texAddon=AddonId, texFile="img/textures/rgBlackBar.png", alpha=.5,
				backgroundColor={r=0.08, g=0.91, b=1, a=0}
			},
			{
				id="ttLabelHealthAndAbsorbPercent", type="Label", parent="frame", layer=80,
				attach = {{ point="CENTERLEFT", element="ttBarAbsorb", targetPoint="CENTERLEFT", offsetX=2.5, offsetY=0 }},
				visibilityBinding="id", 
				text="( {absorb} )", default="", color={r=1, g=1, b=1, a=1}, font="ArmWrestler",  fontSize=10, outline=true,
			},
			{
				id="ttLabelHealth", type="Label", parent="frame", layer=80, alpha=1,
				attach = {{ point="CENTERRIGHT", element="ttBarHealth", targetPoint="CENTERRIGHT", offsetX=-2, offsetY=0 }},
				visibilityBinding="healthText",
				text="{health} ({healthPercent}%)", default="", color={r=1, g=1, b=1, a=1}, font="ArmWrestler", fontSize=10,
			},
			{
				id="ttLabelStatus", type="Label", parent="frame", layer=200,
				attach = {{ point="CENTER", element="ttBarHealth", targetPoint="CENTER", offsetX=0, offsetY=0 }},
				visibilityBinding="rgStatus",
				text="{rgStatus}", default="", font="Bazooka", fontSize=12
			},
			{
				id="ttLabelName", type="Label", parent="frame", layer=200, alpha=1,
				attach = {{ point="BOTTOMCENTER", element="ttInnerFrame", targetPoint="TOPCENTER", offsetX=1, offsetY=4 }},
				visibilityBinding="name",
				text="{nameShort}", default="", color={r=1,g=1,b=1,a=1}, font="ArmWrestler", fontSize=12, outline=true
			},
			{
				id="ttLabelRange", type="Label", parent="frame", layer=200, alpha=1,
				attach = {{ point="BOTTOMCENTER", element="ttFrameBackdrop", targetPoint="BOTTOMCENTER", offsetX=0, offsetY=-.5 }},
				visibilityBinding="rangeDisplay",
				text="{range} m", default="", colorBinding="rangeColor", font="ArmWrestler", fontSize=12, 
			},
			{
				id="ttImgRole", type="MediaSet", parent="frame", layer=80, alpha=1,
				attach = {{ point="CENTER", element="ttFrameBackdrop", targetPoint="BOTTOMLEFT", offsetX=4, offsetY=-4 }},
				width = 18, height = 18,
				visibilityBinding="role",
				nameBinding="role", 
				names = { ["tank"] = "rgRaidTank", ["heal"] = "rgRaidHeal", ["dps"] = "rgRaidDPS", ["support"] = "rgRaidSupport" },
			},
			{
				id="ttImgMark", type="MediaSet", parent="frame", layer=180,
				attach = 
				{
					{ point="CENTER", element="ttFrameBackdrop", targetPoint="TOPRIGHT", offsetX=-4, offsetY=4 },
				},
				width = 20, height = 20,
				nameBinding="mark",
				names = 
					{
						["1"] = "riftMark01",
						["2"] = "riftMark02",
						["3"] = "riftMark03",
						["4"] = "riftMark04",
						["5"] = "riftMark05",
						["6"] = "riftMark06",
						["7"] = "riftMark07",
						["8"] = "riftMark08",
						["9"] = "riftMark09",
						["10"] = "riftMark10",
						["11"] = "riftMark11",
						["12"] = "riftMark12",
						["13"] = "riftMark13",
						["14"] = "riftMark14",
						["15"] = "riftMark15",
						["16"] = "riftMark16",
						["17"] = "riftMark17",
					},
				visibilityBinding="mark", alpha=1.0,
			},
			{
				id="ttImgLevel", type="MediaSet", parent="frame", layer=160, alpha=1,
				attach = {{ point="CENTER", element="ttFrameBackdrop", targetPoint="TOPLEFT", offsetX=-2, offsetY=2 }},
				width=34, height=34,
				visibilityBinding="levelBorder",
				nameBinding="levelBorder",
				names =
				{
						["eliteIcon"] = "eliteIcon",
						["normalIcon"] = "normalIcon",
				},
			},
			{
				id="ttImgLevelBoss", type="Image", parent="frame", layer=170,
				attach = {{ point="CENTER", element="ttImgLevel", targetPoint="CENTER", offsetX=0, offsetY=-0 }},
				texAddon=AddonId, texFile="img/icons/bossIcon.png", alpha=1,
				visibilityBinding="idBoss",
				width=34, height=34,
			},
			{
				id="ttLevelLabel", type="Label", parent="frame", layer=180,
				attach = {{ point="CENTER", element="ttImgLevel", targetPoint="CENTER", offsetX=0, offsetY=0 }},
				visibilityBinding="levelText",
				colorBinding="difficultyColor", 
				text="{level}", default="", font="ArmWrestler",  fontSize=12
			},				
			{
				id="ttImgLevelUnknown", type="Image", parent="frame", layer=200,
				attach = {{ point="CENTER", element="ttImgLevel", targetPoint="CENTER", offsetX=0, offsetY=-1 }},
				texAddon=AddonId, texFile="img/icons/levelUnknown.png",
				visibilityBinding="levelUnknown",
				width=28, height=28,
			},
			{
				id="ttImgRare", type="Image", parent="frame", layer=60,
				attach = {{ point="CENTER", element="ttFrameBackdrop", targetPoint="TOPCENTER", offsetX=0, offsetY=0 }},
				texAddon=AddonId, texFile="img/icons/blueDiamond.png",
				visibilityBinding="guaranteedLoot",
				width=32, height=32,
			},
			--[[
			{
				id="buffPanelDebuffs", type="BuffPanel", parent="frame", layer=90,
				attach = {{ point="BOTTOMRIGHT", element="ttFrameBackdrop", targetPoint="TOPRIGHT", offsetX=-4, offsetY=-24 }},
				rows=2, cols=4, iconSize=28, iconSpacingHorizontal=1, iconSpacingVertical=14, borderThickness=1, 
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=false, acceptHighPriorityBuffs=false, acceptCriticalPriorityBuffs=false,
				acceptLowPriorityDebuffs=true, acceptMediumPriorityDebuffs=true, acceptHighPriorityDebuffs=true, acceptCriticalPriorityDebuffs=true,
				growthDirection = "left_up", borderColor={r=1,g=1,b=1,a=1},
				timerSize=12, timerOffsetX=0, timerOffsetY=22, timerBackgroundColor={r=0,g=0,b=0,a=0.2},
				stackSize=12, stackOffsetX=0, stackOffsetY=0, stackBackgroundColor={r=0,g=0,b=0,a=0.3},
				sweepOverlay=false,
			},
			{
				id="buffPanelBuffs", type="BuffPanel", parent="HorizontalBar", layer=90,
				attach = {{ point="TOPLEFT", element="ttFrameBackdrop", targetPoint="BOTTOMLEFT", offsetX=0, offsetY=6 }},
				rows=2, cols=5, iconSize=22, iconSpacingHorizontal=1, iconSpacingVertical=14, borderThickness=1,
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=true, acceptHighPriorityBuffs=true, acceptCriticalPriorityBuffs=true,
				acceptLowPriorityDebuffs=false, acceptMediumPriorityDebuffs=false, acceptHighPriorityDebuffs=false, acceptCriticalPriorityDebuffs=false,
				growthDirection = "right_down", selfCast=false,
				timerSize=10, timerOffsetX=0, timerOffsetY=20, timerBackgroundColor={r=0,g=0,b=0,a=0.2},
				stackSize=20, stackOffsetX=0, stackOffsetY=0, stackBackgroundColor={r=0,g=0,b=0,a=0.3},
				borderColor={r=0,g=0,b=0,a=1},
				sweepOverlay=false,
			},
			--]]
			{
				id="ttFrameBlocked", type="Frame", parent="frame", layer=6, visibilityBinding="blockedOrOutOfRange",
				color={r=0,g=0,b=0},alpha=0.6,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=11, offsetY=-6 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-11, offsetY=6 },  
				},
			},
			{
				id="ttFrameBlocked2", type="Frame", parent="frame", layer=100, visibilityBinding="blockedOrOutOfRange",
				color={r=0,g=0,b=0},alpha=0.6,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=-11, offsetY=13 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=11, offsetY=-13.75 },
				},
			},
		}
	}
	
	if WT.UnitFrame.EnableResizableTemplate then
            WT.UnitFrame.EnableResizableTemplate(self, rgTTFrameWidth, rgTTFrameHeight, template.elements)
    end
	
	for idx,element in ipairs(template.elements) do
		local showElement = true
		if options.excludeBuffs and element.type=="BuffPanel" then showElement = false end
		if options.excludeCasts and ((element.id == "barCast") or (element.id == "labelCast")) then showElement = false end
		if not options.showAbsorb and (element.id=="barAbsorb" or element.id=="labelHealthAndAbsorbPercent") then 
			showElement = false 
		end
		if showElement then
			self:CreateElement(element)
		end 
	end
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
end

